# *********************************************************************#
# Bladed Python Models API                                             #
# Copyright (c) DNV Services UK Limited (c) 2025. All rights reserved. #
# MIT License (see license file)                                       #
# *********************************************************************#


# coding: utf-8

from __future__ import annotations

from datetime import date, datetime  # noqa: F401
from enum import Enum, IntEnum

import os
import re  # noqa: F401
from typing import Annotated, Any, Dict, List, Literal, Optional, Set, Type, Union, Callable, Iterable  # noqa: F401
from pathlib import Path
from typing import TypeVar
Model = TypeVar('Model', bound='BaseModel')
StrBytes = Union[str, bytes]

from pydantic import AnyUrl, BaseModel, EmailStr, Field, validator, root_validator, Extra,PrivateAttr  # noqa: F401
from pydantic import ValidationError
from pydantic.error_wrappers import ErrorWrapper
from pydantic.utils import ROOT_KEY
from json import encoder
from dnv_bladed_models.bladed_model import BladedModel
from dnv_bladed_models.load_bank import LoadBank

from .schema_helper import SchemaHelper
from .models_impl import *


class YawActuation(BladedModel):
    r"""
    The properties related to the actuation of the yaw system.
    
    Attributes
    ----------
    ExternalLoadsFilepath : str
        The filepath to an External Loads DLL module that will apply the relevant actuator torque. This is optional and can also be applied in other specified External Loads DLLs.
    
    EffectiveGearRatio : float
        The effective total ratio for all load banks of the input high-speed shaft's rotations to the output low-speed shaft's rotations.
    
    LoadBanks : List[LoadBank]
        The properties of the load banks.
    
    Notes
    -----
    
    """
    ExternalLoadsFilepath: str = Field(alias="ExternalLoadsFilepath", default=None)
    EffectiveGearRatio: float = Field(alias="EffectiveGearRatio", default=None)
    LoadBanks: List[LoadBank] = Field(alias="LoadBanks", default=list())

    _relative_schema_path = 'Components/YawSystem/YawActuation/YawActuation.json'
    _type_info = TypeInfo(
        set([]),
        set([]),
        set(['LoadBanks',]),
        None).merge(BladedModel._type_info)


    class Config:
        extra = Extra.forbid
        validate_assignment = True
        allow_population_by_field_name = True
        pass

    def _entity(self) -> bool:
        return True


YawActuation.update_forward_refs()



# *********************************************************************#
# Bladed Python Models API                                             #
# Copyright (c) DNV Services UK Limited (c) 2025. All rights reserved. #
# MIT License (see license file)                                       #
# *********************************************************************#


# coding: utf-8

from __future__ import annotations

from abc import ABC

from datetime import date, datetime  # noqa: F401
from enum import Enum, IntEnum

import os
import re  # noqa: F401
from typing import Annotated, Any, Dict, List, Literal, Optional, Set, Type, Union, Callable, Iterable  # noqa: F401
from pathlib import Path
from typing import TypeVar
Model = TypeVar('Model', bound='BaseModel')
StrBytes = Union[str, bytes]

from pydantic import AnyUrl, BaseModel, EmailStr, Field, validator, root_validator, Extra,PrivateAttr  # noqa: F401
from pydantic import ValidationError
from pydantic.error_wrappers import ErrorWrapper
from pydantic.utils import ROOT_KEY
from json import encoder
from dnv_bladed_models.bladed_model import BladedModel
class AddedInertia_AddedInertiaTypeEnum(str, Enum):
    POINT_INERTIA = "PointInertia"
    SIX_BY_SIX_INERTIA = "SixBySixInertia"
    INSERT = "Insert"

from .schema_helper import SchemaHelper
from .models_impl import *


class AddedInertia(BladedModel, ABC):
    r"""
    The common properties of inertias added to the tower component.
    
    Attributes
    ----------
    AddedInertiaType : AddedInertia_AddedInertiaTypeEnum
        Defines the specific type of model in use.
    
    IgnoreGravityLoads : bool, default=False
        If true, the loads due to gravity on this inertia will be ignored.
    
    HeightUpTower : float, default=0
        The height measured from the bottom of the tower, assuming that the tower is mounted vertically.
    
    SnapToNearestNode : bool
        If true, the inertias will be added to the nearest structural node, which may be at a significantly different height to that specified.
    
    Notes
    -----
    
    This class is an abstraction, with the following concrete implementations:
        - AddedInertiaInsert
        - PointInertia
        - SixBySixInertia
    
    """
    AddedInertiaType: AddedInertia_AddedInertiaTypeEnum = Field(alias="AddedInertiaType", default=None)
    IgnoreGravityLoads: bool = Field(alias="IgnoreGravityLoads", default=None)
    HeightUpTower: float = Field(alias="HeightUpTower", default=None)
    SnapToNearestNode: bool = Field(alias="SnapToNearestNode", default=None)

    _type_info = TypeInfo(
        set([]),
        set([]),
        set([]),
        None).merge(BladedModel._type_info)


AddedInertia.update_forward_refs()



# *********************************************************************#
# Bladed Python Models API                                             #
# Copyright (c) DNV Services UK Limited (c) 2025. All rights reserved. #
# MIT License (see license file)                                       #
# *********************************************************************#


# coding: utf-8

from __future__ import annotations

from datetime import date, datetime  # noqa: F401
from enum import Enum, IntEnum

import os
import re  # noqa: F401
from typing import Annotated, Any, Dict, List, Literal, Optional, Set, Type, Union, Callable, Iterable  # noqa: F401
from pathlib import Path
from typing import TypeVar
Model = TypeVar('Model', bound='BaseModel')
StrBytes = Union[str, bytes]

from pydantic import AnyUrl, BaseModel, EmailStr, Field, validator, root_validator, Extra,PrivateAttr  # noqa: F401
from pydantic import ValidationError
from pydantic.error_wrappers import ErrorWrapper
from pydantic.utils import ROOT_KEY
from json import encoder
from dnv_bladed_models.dynamic_wake import DynamicWake

from .schema_helper import SchemaHelper
from .models_impl import *


class FreeFlowModel(DynamicWake):
    r"""
    The Free-flow dynamic wake model represents a case where there is no induction, and so no momentum calculation.  The lift, drag and moment polars are used directly.  This could be appropriate to use in e.g. parked or idling.  This option might be removed in a future re-design of the aerodynamic settings.
    
    Attributes
    ----------
    DynamicWakeType : Literal['FreeFlowModel'], default='FreeFlowModel'
        Defines the specific type of DynamicWake model in use.  For a `FreeFlowModel` object, this must always be set to a value of `FreeFlowModel`.
    
    Notes
    -----
    
    """
    DynamicWakeType: Literal['FreeFlowModel'] = Field(alias="DynamicWakeType", default='FreeFlowModel', allow_mutation=False, const=True) # type: ignore

    _relative_schema_path = 'Settings/AerodynamicSettings/AerodynamicModel/MomentumTheoryCorrections/DynamicWake/FreeFlowModel.json'
    _type_info = TypeInfo(
        set([]),
        set([]),
        set([]),
        'DynamicWakeType').merge(DynamicWake._type_info)


    class Config:
        extra = Extra.forbid
        validate_assignment = True
        allow_population_by_field_name = True
        pass

    def _entity(self) -> bool:
        return True


FreeFlowModel.update_forward_refs()



# *********************************************************************#
# Bladed Python Models API                                             #
# Copyright (c) DNV Services UK Limited (c) 2025. All rights reserved. #
# MIT License (see license file)                                       #
# *********************************************************************#


# coding: utf-8

from __future__ import annotations

from abc import ABC

from datetime import date, datetime  # noqa: F401
from enum import Enum, IntEnum

import os
import re  # noqa: F401
from typing import Annotated, Any, Dict, List, Literal, Optional, Set, Type, Union, Callable, Iterable  # noqa: F401
from pathlib import Path
from typing import TypeVar
Model = TypeVar('Model', bound='BaseModel')
StrBytes = Union[str, bytes]

from pydantic import AnyUrl, BaseModel, EmailStr, Field, validator, root_validator, Extra,PrivateAttr  # noqa: F401
from pydantic import ValidationError
from pydantic.error_wrappers import ErrorWrapper
from pydantic.utils import ROOT_KEY
from json import encoder
from dnv_bladed_models.dynamic_upstream_wake import DynamicUpstreamWake
from dnv_bladed_models.exponential_shear_model import ExponentialShearModel
from dnv_bladed_models.gaussian import Gaussian
from dnv_bladed_models.logarithmic_shear_model import LogarithmicShearModel
from dnv_bladed_models.look_up_shear_model import LookUpShearModel
from dnv_bladed_models.preset_wind_direction_transient import PresetWindDirectionTransient
from dnv_bladed_models.steady_wake_deficit import SteadyWakeDeficit
from dnv_bladed_models.steady_wake_deficit_insert import SteadyWakeDeficitInsert
from dnv_bladed_models.user_defined_wake_deficit import UserDefinedWakeDeficit
from dnv_bladed_models.wind import Wind
from dnv_bladed_models.wind_direction_time_history import WindDirectionTimeHistory
from dnv_bladed_models.wind_direction_variation import WindDirectionVariation
from dnv_bladed_models.wind_direction_variation_insert import WindDirectionVariationInsert
from dnv_bladed_models.wind_shear import WindShear
from dnv_bladed_models.wind_shear_insert import WindShearInsert

from .schema_helper import SchemaHelper
from .models_impl import *

TWindShearOptions = TypeVar('TWindShearOptions', ExponentialShearModel, WindShearInsert, LogarithmicShearModel, LookUpShearModel, WindShear, )
TWindDirectionVariationOptions = TypeVar('TWindDirectionVariationOptions', WindDirectionVariationInsert, PresetWindDirectionTransient, WindDirectionTimeHistory, WindDirectionVariation, )
TSteadyWakeDeficitOptions = TypeVar('TSteadyWakeDeficitOptions', Gaussian, SteadyWakeDeficitInsert, UserDefinedWakeDeficit, SteadyWakeDeficit, )

class TimeDomainWind(Wind, ABC):
    r"""
    The definition of a wind field that varies throughout a time domain simulation.
    
    Attributes
    ----------
    ReferenceHeight : float
        The reference height for the wind field, at which all flow conditions are at their nominal values.  If this is omitted, the hub height will be used, and if there is more than one the *highest* hub height.
    
    WindShear : Union[ExponentialShearModel, WindShearInsert, LogarithmicShearModel, LookUpShearModel]
    
    Inclination : float, default=0
        The inclination of the flow relative to the horizontal plane.  Typically this is in the order of 8 degrees for an onshore turbine, and 0 degrees for an offshore turbine.
    
    Direction : float, default=0
        The (constant) direction of the wind relative to the global x-axis.
    
    DirectionVariation : Union[WindDirectionVariationInsert, PresetWindDirectionTransient, WindDirectionTimeHistory]
    
    SteadyWakeDeficit : Union[Gaussian, SteadyWakeDeficitInsert, UserDefinedWakeDeficit]
    
    DynamicUpstreamWake : DynamicUpstreamWake
    
    Notes
    -----
    
    """
    ReferenceHeight: float = Field(alias="ReferenceHeight", default=None)
    WindShear: Union[ExponentialShearModel, WindShearInsert, LogarithmicShearModel, LookUpShearModel] = Field(alias="WindShear", default=None, discriminator='WindShearType')
    Inclination: float = Field(alias="Inclination", default=None)
    Direction: float = Field(alias="Direction", default=None)
    DirectionVariation: Union[WindDirectionVariationInsert, PresetWindDirectionTransient, WindDirectionTimeHistory] = Field(alias="DirectionVariation", default=None, discriminator='DirectionVariationType')
    SteadyWakeDeficit: Union[Gaussian, SteadyWakeDeficitInsert, UserDefinedWakeDeficit] = Field(alias="SteadyWakeDeficit", default=None, discriminator='SteadyWakeDeficitType')
    DynamicUpstreamWake: DynamicUpstreamWake = Field(alias="DynamicUpstreamWake", default=None)

    _type_info = TypeInfo(
        set([('WindShear', 'WindShearType'),('DirectionVariation', 'DirectionVariationType'),('SteadyWakeDeficit', 'SteadyWakeDeficitType'),]),
        set([]),
        set([]),
        None).merge(Wind._type_info)


    @property
    def WindShear_as_ExponentialShearModel(self) -> ExponentialShearModel:
        """
        Retrieves the value of WindShear guaranteeing it is a ExponentialShearModel; if it is not, an error is raised.

        Useful when using type-checking development tools, as it provides a concise way to retrieve a value and assign a variable with the correct type.

        Returns
        -------
        ExponentialShearModel
            A model object, guaranteed to be a ExponentialShearModel.

        Raises
        ------
        TypeError
            If the value is not a ExponentialShearModel.
        """
        return self.WindShear_as(ExponentialShearModel)


    @property
    def WindShear_as_LogarithmicShearModel(self) -> LogarithmicShearModel:
        """
        Retrieves the value of WindShear guaranteeing it is a LogarithmicShearModel; if it is not, an error is raised.

        Useful when using type-checking development tools, as it provides a concise way to retrieve a value and assign a variable with the correct type.

        Returns
        -------
        LogarithmicShearModel
            A model object, guaranteed to be a LogarithmicShearModel.

        Raises
        ------
        TypeError
            If the value is not a LogarithmicShearModel.
        """
        return self.WindShear_as(LogarithmicShearModel)


    @property
    def WindShear_as_LookUpShearModel(self) -> LookUpShearModel:
        """
        Retrieves the value of WindShear guaranteeing it is a LookUpShearModel; if it is not, an error is raised.

        Useful when using type-checking development tools, as it provides a concise way to retrieve a value and assign a variable with the correct type.

        Returns
        -------
        LookUpShearModel
            A model object, guaranteed to be a LookUpShearModel.

        Raises
        ------
        TypeError
            If the value is not a LookUpShearModel.
        """
        return self.WindShear_as(LookUpShearModel)


    @property
    def WindShear_as_inline(self) -> Union[ExponentialShearModel, LogarithmicShearModel, LookUpShearModel]:
        """
        Retrieves the value of WindShear as a model object; if the value is specified with a '$insert', an error is raised.

        Returns
        -------
        Union[ExponentialShearModel, LogarithmicShearModel, LookUpShearModel]
            A model object at the specified index, guaranteed to not be a '$insert'.

        Raises
        ------
        TypeError
            If the value is not one of the concrete types of WindShear; i.e. it is specified with a '$insert'.
        """
        if isinstance(self.WindShear, WindShearInsert) or self.WindShear.is_insert:
            raise TypeError(f"Expected WindShear value to be an in-line object, but it is currently in the '$insert' state.")
        return self.WindShear


    def WindShear_as(self, cls: Type[TWindShearOptions])-> TWindShearOptions:
        """
        Retrieves the value of WindShear, ensuring it is of the specified type.
        The specified type must be one of the valid concrete types of WindShear, or the '$insert' type
        (in the case the model is defined to be inserted from an external resource).

        Useful when using type-checking development tools, as it provides a concise way to access a value with the correct type.

        Parameters
        ----------
        cls: Type[Union[ExponentialShearModel, WindShearInsert, LogarithmicShearModel, LookUpShearModel]]
            One of the valid concrete types of WindShear, or the '$insert' type
            (in the case the model is defined to be inserted from an external resource).

        Returns
        -------
        TWindShearOptions
            A model object of the specified type.

        Raises
        ------
        TypeError
            If the value is not of the specified type.

        Examples
        --------
        Get a reference to the value when it is one of the types of WindShear:
        >>> val_obj = model_obj.WindShear_as(models.ExponentialShearModel)
        >>> val_obj = model_obj.WindShear_as(models.LogarithmicShearModel)
        >>> val_obj = model_obj.WindShear_as(models.LookUpShearModel)

        Get a reference to the value, when it was specified with a '$insert' and read in from a file:
        >>> insert_obj = model_obj.WindShear_as(models.WindShearInsert)
        """
        if not isinstance(self.WindShear, cls):
            raise TypeError(f"Expected WindShear of type '{cls.__name__}' but was type '{type(self.WindShear).__name__}'")
        return self.WindShear


    @property
    def DirectionVariation_as_PresetWindDirectionTransient(self) -> PresetWindDirectionTransient:
        """
        Retrieves the value of DirectionVariation guaranteeing it is a PresetWindDirectionTransient; if it is not, an error is raised.

        Useful when using type-checking development tools, as it provides a concise way to retrieve a value and assign a variable with the correct type.

        Returns
        -------
        PresetWindDirectionTransient
            A model object, guaranteed to be a PresetWindDirectionTransient.

        Raises
        ------
        TypeError
            If the value is not a PresetWindDirectionTransient.
        """
        return self.DirectionVariation_as(PresetWindDirectionTransient)


    @property
    def DirectionVariation_as_WindDirectionTimeHistory(self) -> WindDirectionTimeHistory:
        """
        Retrieves the value of DirectionVariation guaranteeing it is a WindDirectionTimeHistory; if it is not, an error is raised.

        Useful when using type-checking development tools, as it provides a concise way to retrieve a value and assign a variable with the correct type.

        Returns
        -------
        WindDirectionTimeHistory
            A model object, guaranteed to be a WindDirectionTimeHistory.

        Raises
        ------
        TypeError
            If the value is not a WindDirectionTimeHistory.
        """
        return self.DirectionVariation_as(WindDirectionTimeHistory)


    @property
    def DirectionVariation_as_inline(self) -> Union[PresetWindDirectionTransient, WindDirectionTimeHistory]:
        """
        Retrieves the value of DirectionVariation as a model object; if the value is specified with a '$insert', an error is raised.

        Returns
        -------
        Union[PresetWindDirectionTransient, WindDirectionTimeHistory]
            A model object at the specified index, guaranteed to not be a '$insert'.

        Raises
        ------
        TypeError
            If the value is not one of the concrete types of WindDirectionVariation; i.e. it is specified with a '$insert'.
        """
        if isinstance(self.DirectionVariation, WindDirectionVariationInsert) or self.DirectionVariation.is_insert:
            raise TypeError(f"Expected DirectionVariation value to be an in-line object, but it is currently in the '$insert' state.")
        return self.DirectionVariation


    def DirectionVariation_as(self, cls: Type[TWindDirectionVariationOptions])-> TWindDirectionVariationOptions:
        """
        Retrieves the value of DirectionVariation, ensuring it is of the specified type.
        The specified type must be one of the valid concrete types of WindDirectionVariation, or the '$insert' type
        (in the case the model is defined to be inserted from an external resource).

        Useful when using type-checking development tools, as it provides a concise way to access a value with the correct type.

        Parameters
        ----------
        cls: Type[Union[WindDirectionVariationInsert, PresetWindDirectionTransient, WindDirectionTimeHistory]]
            One of the valid concrete types of WindDirectionVariation, or the '$insert' type
            (in the case the model is defined to be inserted from an external resource).

        Returns
        -------
        TWindDirectionVariationOptions
            A model object of the specified type.

        Raises
        ------
        TypeError
            If the value is not of the specified type.

        Examples
        --------
        Get a reference to the value when it is one of the types of WindDirectionVariation:
        >>> val_obj = model_obj.DirectionVariation_as(models.PresetWindDirectionTransient)
        >>> val_obj = model_obj.DirectionVariation_as(models.WindDirectionTimeHistory)

        Get a reference to the value, when it was specified with a '$insert' and read in from a file:
        >>> insert_obj = model_obj.DirectionVariation_as(models.WindDirectionVariationInsert)
        """
        if not isinstance(self.DirectionVariation, cls):
            raise TypeError(f"Expected DirectionVariation of type '{cls.__name__}' but was type '{type(self.DirectionVariation).__name__}'")
        return self.DirectionVariation


    @property
    def SteadyWakeDeficit_as_Gaussian(self) -> Gaussian:
        """
        Retrieves the value of SteadyWakeDeficit guaranteeing it is a Gaussian; if it is not, an error is raised.

        Useful when using type-checking development tools, as it provides a concise way to retrieve a value and assign a variable with the correct type.

        Returns
        -------
        Gaussian
            A model object, guaranteed to be a Gaussian.

        Raises
        ------
        TypeError
            If the value is not a Gaussian.
        """
        return self.SteadyWakeDeficit_as(Gaussian)


    @property
    def SteadyWakeDeficit_as_UserDefinedWakeDeficit(self) -> UserDefinedWakeDeficit:
        """
        Retrieves the value of SteadyWakeDeficit guaranteeing it is a UserDefinedWakeDeficit; if it is not, an error is raised.

        Useful when using type-checking development tools, as it provides a concise way to retrieve a value and assign a variable with the correct type.

        Returns
        -------
        UserDefinedWakeDeficit
            A model object, guaranteed to be a UserDefinedWakeDeficit.

        Raises
        ------
        TypeError
            If the value is not a UserDefinedWakeDeficit.
        """
        return self.SteadyWakeDeficit_as(UserDefinedWakeDeficit)


    @property
    def SteadyWakeDeficit_as_inline(self) -> Union[Gaussian, UserDefinedWakeDeficit]:
        """
        Retrieves the value of SteadyWakeDeficit as a model object; if the value is specified with a '$insert', an error is raised.

        Returns
        -------
        Union[Gaussian, UserDefinedWakeDeficit]
            A model object at the specified index, guaranteed to not be a '$insert'.

        Raises
        ------
        TypeError
            If the value is not one of the concrete types of SteadyWakeDeficit; i.e. it is specified with a '$insert'.
        """
        if isinstance(self.SteadyWakeDeficit, SteadyWakeDeficitInsert) or self.SteadyWakeDeficit.is_insert:
            raise TypeError(f"Expected SteadyWakeDeficit value to be an in-line object, but it is currently in the '$insert' state.")
        return self.SteadyWakeDeficit


    def SteadyWakeDeficit_as(self, cls: Type[TSteadyWakeDeficitOptions])-> TSteadyWakeDeficitOptions:
        """
        Retrieves the value of SteadyWakeDeficit, ensuring it is of the specified type.
        The specified type must be one of the valid concrete types of SteadyWakeDeficit, or the '$insert' type
        (in the case the model is defined to be inserted from an external resource).

        Useful when using type-checking development tools, as it provides a concise way to access a value with the correct type.

        Parameters
        ----------
        cls: Type[Union[Gaussian, SteadyWakeDeficitInsert, UserDefinedWakeDeficit]]
            One of the valid concrete types of SteadyWakeDeficit, or the '$insert' type
            (in the case the model is defined to be inserted from an external resource).

        Returns
        -------
        TSteadyWakeDeficitOptions
            A model object of the specified type.

        Raises
        ------
        TypeError
            If the value is not of the specified type.

        Examples
        --------
        Get a reference to the value when it is one of the types of SteadyWakeDeficit:
        >>> val_obj = model_obj.SteadyWakeDeficit_as(models.Gaussian)
        >>> val_obj = model_obj.SteadyWakeDeficit_as(models.UserDefinedWakeDeficit)

        Get a reference to the value, when it was specified with a '$insert' and read in from a file:
        >>> insert_obj = model_obj.SteadyWakeDeficit_as(models.SteadyWakeDeficitInsert)
        """
        if not isinstance(self.SteadyWakeDeficit, cls):
            raise TypeError(f"Expected SteadyWakeDeficit of type '{cls.__name__}' but was type '{type(self.SteadyWakeDeficit).__name__}'")
        return self.SteadyWakeDeficit


TimeDomainWind.update_forward_refs()



# *********************************************************************#
# Bladed Python Models API                                             #
# Copyright (c) DNV Services UK Limited (c) 2025. All rights reserved. #
# MIT License (see license file)                                       #
# *********************************************************************#


# coding: utf-8

from __future__ import annotations

from datetime import date, datetime  # noqa: F401
from enum import Enum, IntEnum

import os
import re  # noqa: F401
from typing import Annotated, Any, Dict, List, Literal, Optional, Set, Type, Union, Callable, Iterable  # noqa: F401
from pathlib import Path
from typing import TypeVar
Model = TypeVar('Model', bound='BaseModel')
StrBytes = Union[str, bytes]

from pydantic import AnyUrl, BaseModel, EmailStr, Field, validator, root_validator, Extra,PrivateAttr  # noqa: F401
from pydantic import ValidationError
from pydantic.error_wrappers import ErrorWrapper
from pydantic.utils import ROOT_KEY
from json import encoder
from dnv_bladed_models.bladed_model import BladedModel
from dnv_bladed_models.full_load_operation import FullLoadOperation
from dnv_bladed_models.nominal_hub_wind_speed_vs_minimum_steady_state_pitch_angle import NominalHubWindSpeedVsMinimumSteadyStatePitchAngle
from dnv_bladed_models.partial_load_operation import PartialLoadOperation

from .schema_helper import SchemaHelper
from .models_impl import *


class VariableSpeedPitchRegulatedControlModel(BladedModel):
    r"""
    The operational parameters that Bladed uses to determine the steady state conditions of the turbine. Although some of the values will be made available to the external controller through the API, they will not be used by Bladed during the remainder of a time domain simulation.
    
    Attributes
    ----------
    MinimumSteadyStatePitchAngle : float
        This value represents the minimum allowable blade pitch angle used for determining pitch angle during steady state/initial condition calculation. It is combined with the MaximumSteadyStatePitchAngle to define the allowable pitch range. Additionally, the external controller API can access this value.
    
    MaximumSteadyStatePitchAngle : float
        This value represents the minimum allowable blade pitch angle used for determining pitch angle during steady state/initial condition calculation. It is combined with the MinimumSteadyStatePitchAngle to define the allowable pitch range. Please note that the steady-state algorithm does not support overspeed steady state conditions in cases where rotor speeds exceed the specified value for FullLoadOperation::GeneratorSpeed. Additionally, this value is accessible via the external controller API.
    
    NominalHubWindSpeedVsMinimumSteadyStatePitchAngle : List[NominalHubWindSpeedVsMinimumSteadyStatePitchAngle]
        The look-up table for specifying hub wind speed against the respective minimum pitch angle for pitch scheduling. Wind speeds need to be in monotonic ascending order. Linear interpolation is used between points. If the hub wind speed exceeds the specified value the nearest will be used and a warning message will be written.
    
    PartialLoadOperation : PartialLoadOperation
    
    FullLoadOperation : FullLoadOperation
    
    Notes
    -----
    
    """
    MinimumSteadyStatePitchAngle: float = Field(alias="MinimumSteadyStatePitchAngle", default=None)
    MaximumSteadyStatePitchAngle: float = Field(alias="MaximumSteadyStatePitchAngle", default=None)
    NominalHubWindSpeedVsMinimumSteadyStatePitchAngle: List[NominalHubWindSpeedVsMinimumSteadyStatePitchAngle] = Field(alias="NominalHubWindSpeedVsMinimumSteadyStatePitchAngle", default=list())
    PartialLoadOperation: PartialLoadOperation = Field(alias="PartialLoadOperation", default=None)
    FullLoadOperation: FullLoadOperation = Field(alias="FullLoadOperation", default=None)

    _relative_schema_path = 'Turbine/TurbineOperationalParameters/VariableSpeedPitchRegulatedControlModel/VariableSpeedPitchRegulatedControlModel.json'
    _type_info = TypeInfo(
        set([]),
        set([]),
        set(['NominalHubWindSpeedVsMinimumSteadyStatePitchAngle',]),
        None).merge(BladedModel._type_info)


    class Config:
        extra = Extra.forbid
        validate_assignment = True
        allow_population_by_field_name = True
        pass

    def _entity(self) -> bool:
        return True


VariableSpeedPitchRegulatedControlModel.update_forward_refs()



# *********************************************************************#
# Bladed Python Models API                                             #
# Copyright (c) DNV Services UK Limited (c) 2025. All rights reserved. #
# MIT License (see license file)                                       #
# *********************************************************************#


# coding: utf-8

from __future__ import annotations

from datetime import date, datetime  # noqa: F401
from enum import Enum, IntEnum

import os
import re  # noqa: F401
from typing import Annotated, Any, Dict, List, Literal, Optional, Set, Type, Union, Callable, Iterable  # noqa: F401
from pathlib import Path
from typing import TypeVar
Model = TypeVar('Model', bound='BaseModel')
StrBytes = Union[str, bytes]

from pydantic import AnyUrl, BaseModel, EmailStr, Field, validator, root_validator, Extra,PrivateAttr  # noqa: F401
from pydantic import ValidationError
from pydantic.error_wrappers import ErrorWrapper
from pydantic.utils import ROOT_KEY
from json import encoder
from dnv_bladed_models.component import Component
from dnv_bladed_models.friction import Friction
from dnv_bladed_models.idealised_pitch_actuator import IdealisedPitchActuator
from dnv_bladed_models.linear_pitch_actuator import LinearPitchActuator
from dnv_bladed_models.pitch_actuator import PitchActuator
from dnv_bladed_models.pitch_actuator_insert import PitchActuatorInsert
from dnv_bladed_models.pitch_controller import PitchController
from dnv_bladed_models.pitch_end_stops import PitchEndStops
from dnv_bladed_models.pitch_limit_switches import PitchLimitSwitches
from dnv_bladed_models.pitch_system_connectable_nodes import PitchSystemConnectableNodes
from dnv_bladed_models.pitch_system_output_group_library import PitchSystemOutputGroupLibrary
from dnv_bladed_models.pitch_system_rotary_drive import PitchSystemRotaryDrive

from .schema_helper import SchemaHelper
from .models_impl import *

TPitchActuatorOptions = TypeVar('TPitchActuatorOptions', IdealisedPitchActuator, PitchActuatorInsert, LinearPitchActuator, PitchSystemRotaryDrive, PitchActuator, )

class PitchSystem(Component):
    r"""
    A pitch system, including bearing, actuation, and independent control system.
    
    Attributes
    ----------
    ComponentType : Literal['PitchSystem'], default='PitchSystem'
        Defines the specific type of Component model in use.  For a `PitchSystem` object, this must always be set to a value of `PitchSystem`.
    
    PitchController : PitchController
    
    LimitSwitches : PitchLimitSwitches
    
    EndStops : PitchEndStops
    
    Bearing : Friction
    
    Actuator : Union[IdealisedPitchActuator, PitchActuatorInsert, LinearPitchActuator, PitchSystemRotaryDrive]
    
    OutputGroups : PitchSystemOutputGroupLibrary, Not supported yet
    
    ConnectableNodes : PitchSystemConnectableNodes, Not supported yet
    
    Notes
    -----
    
    """
    ComponentType: Literal['PitchSystem'] = Field(alias="ComponentType", default='PitchSystem', allow_mutation=False, const=True) # type: ignore
    PitchController: PitchController = Field(alias="PitchController", default=None)
    LimitSwitches: PitchLimitSwitches = Field(alias="LimitSwitches", default=None)
    EndStops: PitchEndStops = Field(alias="EndStops", default=None)
    Bearing: Friction = Field(alias="Bearing", default=None)
    Actuator: Union[IdealisedPitchActuator, PitchActuatorInsert, LinearPitchActuator, PitchSystemRotaryDrive] = Field(alias="Actuator", default=None, discriminator='ActuatorDriveType')
    OutputGroups: PitchSystemOutputGroupLibrary = Field(alias="OutputGroups", default=PitchSystemOutputGroupLibrary()) # Not supported yet
    ConnectableNodes: PitchSystemConnectableNodes = Field(alias="ConnectableNodes", default=PitchSystemConnectableNodes()) # Not supported yet

    _relative_schema_path = 'Components/PitchSystem/PitchSystem.json'
    _type_info = TypeInfo(
        set([('Actuator', 'ActuatorDriveType'),]),
        set([]),
        set(['OutputGroups','ConnectableNodes',]),
        'ComponentType').merge(Component._type_info)


    class Config:
        extra = Extra.forbid
        validate_assignment = True
        allow_population_by_field_name = True
        pass

    @property
    def Actuator_as_IdealisedPitchActuator(self) -> IdealisedPitchActuator:
        """
        Retrieves the value of Actuator guaranteeing it is a IdealisedPitchActuator; if it is not, an error is raised.

        Useful when using type-checking development tools, as it provides a concise way to retrieve a value and assign a variable with the correct type.

        Returns
        -------
        IdealisedPitchActuator
            A model object, guaranteed to be a IdealisedPitchActuator.

        Raises
        ------
        TypeError
            If the value is not a IdealisedPitchActuator.
        """
        return self.Actuator_as(IdealisedPitchActuator)


    @property
    def Actuator_as_LinearPitchActuator(self) -> LinearPitchActuator:
        """
        Retrieves the value of Actuator guaranteeing it is a LinearPitchActuator; if it is not, an error is raised.

        Useful when using type-checking development tools, as it provides a concise way to retrieve a value and assign a variable with the correct type.

        Returns
        -------
        LinearPitchActuator
            A model object, guaranteed to be a LinearPitchActuator.

        Raises
        ------
        TypeError
            If the value is not a LinearPitchActuator.
        """
        return self.Actuator_as(LinearPitchActuator)


    @property
    def Actuator_as_PitchSystemRotaryDrive(self) -> PitchSystemRotaryDrive:
        """
        Retrieves the value of Actuator guaranteeing it is a PitchSystemRotaryDrive; if it is not, an error is raised.

        Useful when using type-checking development tools, as it provides a concise way to retrieve a value and assign a variable with the correct type.

        Returns
        -------
        PitchSystemRotaryDrive
            A model object, guaranteed to be a PitchSystemRotaryDrive.

        Raises
        ------
        TypeError
            If the value is not a PitchSystemRotaryDrive.
        """
        return self.Actuator_as(PitchSystemRotaryDrive)


    @property
    def Actuator_as_inline(self) -> Union[IdealisedPitchActuator, LinearPitchActuator, PitchSystemRotaryDrive]:
        """
        Retrieves the value of Actuator as a model object; if the value is specified with a '$insert', an error is raised.

        Returns
        -------
        Union[IdealisedPitchActuator, LinearPitchActuator, PitchSystemRotaryDrive]
            A model object at the specified index, guaranteed to not be a '$insert'.

        Raises
        ------
        TypeError
            If the value is not one of the concrete types of PitchActuator; i.e. it is specified with a '$insert'.
        """
        if isinstance(self.Actuator, PitchActuatorInsert) or self.Actuator.is_insert:
            raise TypeError(f"Expected Actuator value to be an in-line object, but it is currently in the '$insert' state.")
        return self.Actuator


    def Actuator_as(self, cls: Type[TPitchActuatorOptions])-> TPitchActuatorOptions:
        """
        Retrieves the value of Actuator, ensuring it is of the specified type.
        The specified type must be one of the valid concrete types of PitchActuator, or the '$insert' type
        (in the case the model is defined to be inserted from an external resource).

        Useful when using type-checking development tools, as it provides a concise way to access a value with the correct type.

        Parameters
        ----------
        cls: Type[Union[IdealisedPitchActuator, PitchActuatorInsert, LinearPitchActuator, PitchSystemRotaryDrive]]
            One of the valid concrete types of PitchActuator, or the '$insert' type
            (in the case the model is defined to be inserted from an external resource).

        Returns
        -------
        TPitchActuatorOptions
            A model object of the specified type.

        Raises
        ------
        TypeError
            If the value is not of the specified type.

        Examples
        --------
        Get a reference to the value when it is one of the types of PitchActuator:
        >>> val_obj = model_obj.Actuator_as(models.IdealisedPitchActuator)
        >>> val_obj = model_obj.Actuator_as(models.LinearPitchActuator)
        >>> val_obj = model_obj.Actuator_as(models.PitchSystemRotaryDrive)

        Get a reference to the value, when it was specified with a '$insert' and read in from a file:
        >>> insert_obj = model_obj.Actuator_as(models.PitchActuatorInsert)
        """
        if not isinstance(self.Actuator, cls):
            raise TypeError(f"Expected Actuator of type '{cls.__name__}' but was type '{type(self.Actuator).__name__}'")
        return self.Actuator


    def _entity(self) -> bool:
        return True


PitchSystem.update_forward_refs()



# *********************************************************************#
# Bladed Python Models API                                             #
# Copyright (c) DNV Services UK Limited (c) 2025. All rights reserved. #
# MIT License (see license file)                                       #
# *********************************************************************#


# coding: utf-8

from __future__ import annotations

from datetime import date, datetime  # noqa: F401
from enum import Enum, IntEnum

import os
import re  # noqa: F401
from typing import Annotated, Any, Dict, List, Literal, Optional, Set, Type, Union, Callable, Iterable  # noqa: F401
from pathlib import Path
from typing import TypeVar
Model = TypeVar('Model', bound='BaseModel')
StrBytes = Union[str, bytes]

from pydantic import AnyUrl, BaseModel, EmailStr, Field, validator, root_validator, Extra,PrivateAttr  # noqa: F401
from pydantic import ValidationError
from pydantic.error_wrappers import ErrorWrapper
from pydantic.utils import ROOT_KEY
from json import encoder
from dnv_bladed_models.blade_modelling import BladeModelling
from dnv_bladed_models.structural_mode import StructuralMode

from .schema_helper import SchemaHelper
from .models_impl import *


class ModalBladeModelling(BladeModelling):
    r"""
    The properties for the structural modelling of the blade using modal reduction.
    
    Attributes
    ----------
    BladeModellingType : Literal['ModalBladeModelling'], default='ModalBladeModelling'
        Defines the specific type of BladeModelling model in use.  For a `ModalBladeModelling` object, this must always be set to a value of `ModalBladeModelling`.
    
    WholeBladeModeDampingRatios : List[StructuralMode]
        List of known whole-blade mode damping ratios. If the list is incomplete, meaning it lacks damping ratios for all modes, Bladed will assign frequency-proportional damping based on the damping ratio of the highest defined mode. The list must contain at least one entry.
    
    NumberOfModesPerBladePart : int, default=4
        The number of modes used to represent each blade part, or the whole blade if it is represented by a single multibody object.  This should be sufficiently high to capture all of the relevant blade modes, but the higher it is the slower the simulation will become.
    
    Notes
    -----
    
    """
    BladeModellingType: Literal['ModalBladeModelling'] = Field(alias="BladeModellingType", default='ModalBladeModelling', allow_mutation=False, const=True) # type: ignore
    WholeBladeModeDampingRatios: List[StructuralMode] = Field(alias="WholeBladeModeDampingRatios", default=list())
    NumberOfModesPerBladePart: int = Field(alias="NumberOfModesPerBladePart", default=None)

    _relative_schema_path = 'Components/Blade/BladeModelling/ModalBladeModelling.json'
    _type_info = TypeInfo(
        set([]),
        set([]),
        set(['WholeBladeModeDampingRatios',]),
        'BladeModellingType').merge(BladeModelling._type_info)


    class Config:
        extra = Extra.forbid
        validate_assignment = True
        allow_population_by_field_name = True
        pass

    def _entity(self) -> bool:
        return True


ModalBladeModelling.update_forward_refs()



# *********************************************************************#
# Bladed Python Models API                                             #
# Copyright (c) DNV Services UK Limited (c) 2025. All rights reserved. #
# MIT License (see license file)                                       #
# *********************************************************************#


# coding: utf-8

from __future__ import annotations

from datetime import date, datetime  # noqa: F401
from enum import Enum, IntEnum

import os
import re  # noqa: F401
from typing import Annotated, Any, Dict, List, Literal, Optional, Set, Type, Union, Callable, Iterable  # noqa: F401
from pathlib import Path
from typing import TypeVar
Model = TypeVar('Model', bound='BaseModel')
StrBytes = Union[str, bytes]

from pydantic import AnyUrl, BaseModel, EmailStr, Field, validator, root_validator, Extra,PrivateAttr  # noqa: F401
from pydantic import ValidationError
from pydantic.error_wrappers import ErrorWrapper
from pydantic.utils import ROOT_KEY
from json import encoder
from dnv_bladed_models.bladed_model import BladedModel
class ExternalController_CallingConventionEnum(str, Enum):
    CDECL = "__cdecl"
    STDCALL = "__stdcall"
class ExternalController_TimeStepMultiplierEnum(str, Enum):
    EVERY = "Every"
    SECOND = "Second"
    THIRD = "Third"
    FOURTH = "Fourth"
    FIFTH = "Fifth"
    SIXTH = "Sixth"
    SEVENTH = "Seventh"
    EIGTH = "Eigth"
    NINTH = "Ninth"
    TENTH = "Tenth"

from .schema_helper import SchemaHelper
from .models_impl import *


class ExternalController(BladedModel):
    r"""
    A definition of a single controller for the turbine.
    
    Attributes
    ----------
    Filepath : str
        The location of the external controller dll.
    
    CallingConvention : ExternalController_CallingConventionEnum, default='__cdecl'
        The calling convention to be used when calling the external controller.  The default for all C-family languages is '__cdecl'.  The default for FORTRAN is '__stdcall' unless the [C] qualifier is specfied immediately after the function name.  Specifying the wrong calling convention can lead to unexplained system exceptions when attempting to call the external controller.
    
    FunctionName : str, default='ExternalController'
        The name of the function in the dll to run.  This must satisfy the standard external controller typedef, found in the ExternalControllerApi.h.
    
    PassParametersByFile : bool, default=False
        If true, a file will be written containing the parameters in the above box.  The location of this file can be obtained in the external controller using the function GetInfileFilepath.  The name of this file will be \"DISCON.IN\" if there is only one controller, or of the pattern \"DISCONn.IN\", where 'n' is the number of the controller.  If not checked (the default), this string will be directly available using the function GetUserParameters.
    
    ForceLegacy : bool, default=False
        If true, only the old-style 'DISCON' function will be looked for in the controller, and raise an error if it cannot be found.  This is only used for testing legacy controllers where both CONTROLLER and DISCON functions are both defined, but the DISCON function is required.
    
    TimeStepMultiplier : ExternalController_TimeStepMultiplierEnum, default='Every'
        Whether the controller should be called on every discrete timestep, set above.
    
    ParametersAsString : str
        A string that will be passed to the external controller.
    
    ParametersAsJson : Dict[str, Any]
        A JSON object that will be serialised as a string and passed to the external controller.
    
    UseFloatingPointProtection : bool, default=True
        If true, this will apply floating point protection when calling the external controllers.  When the protection is on, any floating point errors are trapped and reported.  When this is switched off, the behaviour will default to that of the computer's floating point machine, but this can often be to not report the error, and to use a semi-random (but often very large) number instead of the correct result.  This can lead to unrepeatable results and numeric errors.
    
    Notes
    -----
    
    """
    Filepath: str = Field(alias="Filepath", default=None)
    CallingConvention: ExternalController_CallingConventionEnum = Field(alias="CallingConvention", default=None)
    FunctionName: str = Field(alias="FunctionName", default=None)
    PassParametersByFile: bool = Field(alias="PassParametersByFile", default=None)
    ForceLegacy: bool = Field(alias="ForceLegacy", default=None)
    TimeStepMultiplier: ExternalController_TimeStepMultiplierEnum = Field(alias="TimeStepMultiplier", default=None)
    ParametersAsString: str = Field(alias="ParametersAsString", default=None)
    ParametersAsJson: Dict[str, Any] = Field(alias="ParametersAsJson", default=None)
    UseFloatingPointProtection: bool = Field(alias="UseFloatingPointProtection", default=None)

    _relative_schema_path = 'Turbine/BladedControl/ExternalController/ExternalController.json'
    _type_info = TypeInfo(
        set([]),
        set([]),
        set([]),
        None).merge(BladedModel._type_info)


    class Config:
        extra = Extra.forbid
        validate_assignment = True
        allow_population_by_field_name = True
        pass

    def _entity(self) -> bool:
        return True


ExternalController.update_forward_refs()



# *********************************************************************#
# Bladed Python Models API                                             #
# Copyright (c) DNV Services UK Limited (c) 2025. All rights reserved. #
# MIT License (see license file)                                       #
# *********************************************************************#


# coding: utf-8

from __future__ import annotations

from datetime import date, datetime  # noqa: F401
from enum import Enum, IntEnum

import os
import re  # noqa: F401
from typing import Annotated, Any, Dict, List, Literal, Optional, Set, Type, Union, Callable, Iterable  # noqa: F401
from pathlib import Path
from typing import TypeVar
Model = TypeVar('Model', bound='BaseModel')
StrBytes = Union[str, bytes]

from pydantic import AnyUrl, BaseModel, EmailStr, Field, validator, root_validator, Extra,PrivateAttr  # noqa: F401
from pydantic import ValidationError
from pydantic.error_wrappers import ErrorWrapper
from pydantic.utils import ROOT_KEY
from json import encoder
from dnv_bladed_models.steady_calculation import SteadyCalculation
from dnv_bladed_models.steady_calculation_outputs import SteadyCalculationOutputs
from dnv_bladed_models.velocity_range import VelocityRange

from .schema_helper import SchemaHelper
from .models_impl import *


class SteadyPowerCurveCalculation(SteadyCalculation):
    r"""
    Defines a calculation which produces power, torque and thrust as a function of wind speed, assuming a uniform steady wind field.  The entire turbine is modelled for this analysis and prebend and sweep included, but without any flexibility.  Most realities such as tower shadow and wind shear are ignored.
    
    Not supported yet.
    
    Attributes
    ----------
    SteadyCalculationType : Literal['SteadyPowerCurve'], default='SteadyPowerCurve', Not supported yet
        Defines the specific type of SteadyCalculation model in use.  For a `SteadyPowerCurve` object, this must always be set to a value of `SteadyPowerCurve`.
    
    WindSpeedRange : VelocityRange
    
    CalculateSpeedAndPitchChange : bool, Not supported yet
        If true, the pitch and speed change schedule will be calculated by Bladed.
    
    PitchAngle : float, Not supported yet
        The pitch angle of all of the turbine blades to be considered for the calculation.
    
    RotorSpeed : float, Not supported yet
        The rotor speed to be considered for the calculation.
    
    Outputs : SteadyCalculationOutputs
    
    Notes
    -----
    This model is not supported yet by the Bladed calculation engine.
    """
    SteadyCalculationType: Literal['SteadyPowerCurve'] = Field(alias="SteadyCalculationType", default='SteadyPowerCurve', allow_mutation=False, const=True) # Not supported yet # type: ignore
    WindSpeedRange: VelocityRange = Field(alias="WindSpeedRange", default=None)
    CalculateSpeedAndPitchChange: bool = Field(alias="CalculateSpeedAndPitchChange", default=None) # Not supported yet
    PitchAngle: float = Field(alias="PitchAngle", default=None) # Not supported yet
    RotorSpeed: float = Field(alias="RotorSpeed", default=None) # Not supported yet
    Outputs: SteadyCalculationOutputs = Field(alias="Outputs", default=None)

    _relative_schema_path = 'SteadyCalculation/SteadyPowerCurveCalculation.json'
    _type_info = TypeInfo(
        set([]),
        set([]),
        set([]),
        'SteadyCalculationType').merge(SteadyCalculation._type_info)


    class Config:
        extra = Extra.forbid
        validate_assignment = True
        allow_population_by_field_name = True
        pass

    def _entity(self) -> bool:
        return True


SteadyPowerCurveCalculation.update_forward_refs()



# *********************************************************************#
# Bladed Python Models API                                             #
# Copyright (c) DNV Services UK Limited (c) 2025. All rights reserved. #
# MIT License (see license file)                                       #
# *********************************************************************#


# coding: utf-8

from __future__ import annotations

from datetime import date, datetime  # noqa: F401
from enum import Enum, IntEnum

import os
import re  # noqa: F401
from typing import Annotated, Any, Dict, List, Literal, Optional, Set, Type, Union, Callable, Iterable  # noqa: F401
from pathlib import Path
from typing import TypeVar
Model = TypeVar('Model', bound='BaseModel')
StrBytes = Union[str, bytes]

from pydantic import AnyUrl, BaseModel, EmailStr, Field, validator, root_validator, Extra,PrivateAttr  # noqa: F401
from pydantic import ValidationError
from pydantic.error_wrappers import ErrorWrapper
from pydantic.utils import ROOT_KEY
from json import encoder
from dnv_bladed_models.bladed_model import BladedModel
from dnv_bladed_models.connectable_node import ConnectableNode

from .schema_helper import SchemaHelper
from .models_impl import *


class ExternalModuleComponentConnectableNodes(BladedModel):
    r"""
    A declaration of what nodes can be connected to.
    
    Not supported yet.
    
    Attributes
    ----------
    Notes
    -----
    This model is not supported yet by the Bladed calculation engine.
    """

    _relative_schema_path = 'Components/ExternalModuleComponent/ExternalModuleComponentConnectableNodes/ExternalModuleComponentConnectableNodes.json'
    _type_info = TypeInfo(
        set([]),
        set([]),
        set([]),
        None).merge(BladedModel._type_info)


    class Config:
        extra = Extra.allow
        validate_assignment = True
        allow_population_by_field_name = True
        pass

    def items(self) -> list[tuple[str, ConnectableNode]]:
        """
        Returns a list of key-value pairs for all of the user-supplied entries currently in the model.
        """
        return [(k, self.__dict__[k]) for k in self.__dict__ if k not in self.__fields__]


    def keys(self) -> list[str]:
        """
        Returns a list of keys for all of the user-supplied entries currently in the model.
        """
        return [k for k in self.__dict__ if k not in self.__fields__]


    def values(self) -> list[ConnectableNode]:
        """
        Returns a list of model objects for all of the user-supplied entries currently in the model.
        """
        return [self.__dict__[k] for k in self.__dict__ if k not in self.__fields__]


    def __len__(self):
        return len([k for k in self.__dict__ if k not in self.__fields__])


    def __contains__(self, item):
        for k in self.__dict__:
            if k not in self.__fields__ and k == item:
                return True
        return False


    def __getitem__(self, key: Union[str, int]) -> ConnectableNode:
        if isinstance(key, int):
            keys = self.keys()
            if len(keys) == 0:
                raise KeyError(f"There are currently no entries in the model object.")
            if key < 0 or key >= len(keys):
                raise KeyError(f"Invalid index specified: {key} (0 >= i < {len(keys)})")
            key = keys[key]
        elif isinstance(key, str):
            if key in self.__fields__:
                raise KeyError(f"Item accessors can only be used for custom entries. '{key}' is a pre-defined model property.")
            if not key in self.__dict__:
                raise KeyError(f"There is no entry with key '{key}'.")
        return getattr(self, key)


    def __setitem__(self, key: str, value: ConnectableNode):
        if not isinstance(key, str):
            raise KeyError(f"Custom entries can only be added with string keys")
        if key in self.__fields__:
            raise KeyError(f"Item accessors can only be used for custom entries. '{key}' is a pre-defined model property.")
        if not isinstance(value, ConnectableNode):
            raise TypeError(f"Entries must be of type 'ConnectableNode'; received '{type(value).__name__}'")
        setattr(self, key, value)


    def __delitem__(self, key: Union[str, int]):
        if isinstance(key, int):
            keys = self.keys()
            if len(keys) == 0:
                raise KeyError(f"There are currently no entries in the model object.")
            if key < 0 or key >= len(keys):
                raise KeyError(f"Invalid index specified: {key} (0 >= i < {len(keys)})")
            key = keys[key]
        elif isinstance(key, str):
            if key in self.__fields__:
                raise KeyError(f"Item accessors can only be used for custom entries. '{key}' is a pre-defined model property.")
            if not key in self.__dict__:
                raise KeyError(f"There is no entry with key '{key}'.")
        delattr(self, key)


    def __setattr__(self, name: str, value: ConnectableNode):
        if not name in self.__fields__ and not isinstance(value, ConnectableNode):
            raise TypeError(f"Entries must be of type 'ConnectableNode'; received '{type(value).__name__}'")
        super().__setattr__(name, value)



    class __CustomEntries__(BaseModel):
        entries: Dict[str, ConnectableNode]


    @classmethod
    def _model_factory(cls: Type['Model'], obj: dict[str, Any]) -> Model:
        return custom_entries_parser(cls, obj, dict, ExternalModuleComponentConnectableNodes.__CustomEntries__)

    def _entity(self) -> bool:
        return True


ExternalModuleComponentConnectableNodes.update_forward_refs()
ExternalModuleComponentConnectableNodes.__CustomEntries__.update_forward_refs()



# *********************************************************************#
# Bladed Python Models API                                             #
# Copyright (c) DNV Services UK Limited (c) 2025. All rights reserved. #
# MIT License (see license file)                                       #
# *********************************************************************#


# coding: utf-8

from __future__ import annotations

from datetime import date, datetime  # noqa: F401
from enum import Enum, IntEnum

import os
import re  # noqa: F401
from typing import Annotated, Any, Dict, List, Literal, Optional, Set, Type, Union, Callable, Iterable  # noqa: F401
from pathlib import Path
from typing import TypeVar
Model = TypeVar('Model', bound='BaseModel')
StrBytes = Union[str, bytes]

from pydantic import AnyUrl, BaseModel, EmailStr, Field, validator, root_validator, Extra,PrivateAttr  # noqa: F401
from pydantic import ValidationError
from pydantic.error_wrappers import ErrorWrapper
from pydantic.utils import ROOT_KEY
from json import encoder
from dnv_bladed_models.brake import Brake
from dnv_bladed_models.brake_insert import BrakeInsert
from dnv_bladed_models.component import Component
from dnv_bladed_models.drivetrain_and_nacelle_mass_properties import DrivetrainAndNacelleMassProperties
from dnv_bladed_models.drivetrain_connectable_nodes import DrivetrainConnectableNodes
from dnv_bladed_models.drivetrain_output_group_library import DrivetrainOutputGroupLibrary
from dnv_bladed_models.high_speed_shaft_flexibility import HighSpeedShaftFlexibility
from dnv_bladed_models.low_speed_shaft import LowSpeedShaft
from dnv_bladed_models.mechanical_losses import MechanicalLosses
from dnv_bladed_models.nacelle_accelerometer import NacelleAccelerometer
from dnv_bladed_models.nacelle_cover import NacelleCover
from dnv_bladed_models.nacelle_inclinometer import NacelleInclinometer
from dnv_bladed_models.nacelle_sensor import NacelleSensor
from dnv_bladed_models.nacelle_sensor_insert import NacelleSensorInsert
from dnv_bladed_models.non_linear_shaft_brake import NonLinearShaftBrake
from dnv_bladed_models.pallet_mounting_flexibility import PalletMountingFlexibility
from dnv_bladed_models.position_of_hub_centre import PositionOfHubCentre
from dnv_bladed_models.simple_shaft_brake import SimpleShaftBrake
from dnv_bladed_models.slipping_clutch import SlippingClutch

from .schema_helper import SchemaHelper
from .models_impl import *

TNacelleSensorOptions = TypeVar('TNacelleSensorOptions', NacelleAccelerometer, NacelleInclinometer, NacelleSensorInsert, NacelleSensor, )
TBrakeOptions = TypeVar('TBrakeOptions', BrakeInsert, NonLinearShaftBrake, SimpleShaftBrake, Brake, )

class DrivetrainAndNacelle(Component):
    r"""
    A drivetrain component.  This includes the gearbox; all shafts and brakes right up to the hub centre; the mainframe of the nacelle; all fairings and ancilliary items on the nacelle.  It excludes the generator.
    
    Attributes
    ----------
    ComponentType : Literal['DrivetrainAndNacelle'], default='DrivetrainAndNacelle'
        Defines the specific type of Component model in use.  For a `DrivetrainAndNacelle` object, this must always be set to a value of `DrivetrainAndNacelle`.
    
    PositionOfHubCentre : PositionOfHubCentre
    
    NacelleCover : NacelleCover
    
    MassProperties : DrivetrainAndNacelleMassProperties
    
    Sensors : List[Union[NacelleAccelerometer, NacelleInclinometer, NacelleSensorInsert]]
        A list of sensors that the controller has access to.
    
    ShaftBrakes : List[Union[BrakeInsert, NonLinearShaftBrake, SimpleShaftBrake]]
        Definitions for the brakes on the various shafts of the drivetrain.
    
    GearboxRatio : float
        The ratio of the high speed shaft (connected to the generator) to the low speed shaft (connected to the hub). Negative values cause the low-speed shaft and high-speed shaft to rotate in opposite directions.
    
    GearboxInertia : float
        The total rotational inertia of the gearbox, referred to the high speed side.
    
    SlippingClutch : SlippingClutch
    
    HighSpeedShaftTorsion : HighSpeedShaftFlexibility
    
    MountingFlexibility : PalletMountingFlexibility
    
    LowSpeedShaft : LowSpeedShaft
    
    Losses : MechanicalLosses
    
    OutputGroups : DrivetrainOutputGroupLibrary, Not supported yet
    
    ConnectableNodes : DrivetrainConnectableNodes, Not supported yet
    
    Notes
    -----
    
    """
    ComponentType: Literal['DrivetrainAndNacelle'] = Field(alias="ComponentType", default='DrivetrainAndNacelle', allow_mutation=False, const=True) # type: ignore
    PositionOfHubCentre: PositionOfHubCentre = Field(alias="PositionOfHubCentre", default=None)
    NacelleCover: NacelleCover = Field(alias="NacelleCover", default=None)
    MassProperties: DrivetrainAndNacelleMassProperties = Field(alias="MassProperties", default=None)
    Sensors: List[Annotated[Union[NacelleAccelerometer, NacelleInclinometer, NacelleSensorInsert], Field(discriminator='NacelleSensorType')]] = Field(alias="Sensors", default=list())
    ShaftBrakes: List[Annotated[Union[BrakeInsert, NonLinearShaftBrake, SimpleShaftBrake], Field(discriminator='BrakeType')]] = Field(alias="ShaftBrakes", default=list())
    GearboxRatio: float = Field(alias="GearboxRatio", default=None)
    GearboxInertia: float = Field(alias="GearboxInertia", default=None)
    SlippingClutch: SlippingClutch = Field(alias="SlippingClutch", default=None)
    HighSpeedShaftTorsion: HighSpeedShaftFlexibility = Field(alias="HighSpeedShaftTorsion", default=None)
    MountingFlexibility: PalletMountingFlexibility = Field(alias="MountingFlexibility", default=None)
    LowSpeedShaft: LowSpeedShaft = Field(alias="LowSpeedShaft", default=None)
    Losses: MechanicalLosses = Field(alias="Losses", default=None)
    OutputGroups: DrivetrainOutputGroupLibrary = Field(alias="OutputGroups", default=DrivetrainOutputGroupLibrary()) # Not supported yet
    ConnectableNodes: DrivetrainConnectableNodes = Field(alias="ConnectableNodes", default=DrivetrainConnectableNodes()) # Not supported yet

    _relative_schema_path = 'Components/DrivetrainAndNacelle/DrivetrainAndNacelle.json'
    _type_info = TypeInfo(
        set([]),
        set([('Sensors', 'NacelleSensorType'),('ShaftBrakes', 'BrakeType'),]),
        set(['Sensors','ShaftBrakes','OutputGroups','ConnectableNodes',]),
        'ComponentType').merge(Component._type_info)


    class Config:
        extra = Extra.forbid
        validate_assignment = True
        allow_population_by_field_name = True
        pass

    @property
    def Sensors_as_inline(self) -> Iterable[Union[NacelleAccelerometer, NacelleInclinometer]]:
        """
        Retrieves the value of Sensors as an iterable of model objects; if any of the values are specified with a '$insert', an error is raised.

        Returns
        -------
        Iterable[Union[NacelleAccelerometer, NacelleInclinometer]]
            A list of model objects, guaranteed to not be a '$insert'.

        Raises
        ------
        TypeError
            If any of the values are not one of the concrete types of NacelleSensor; i.e. they are specified with an 'insert'.

        Examples
        --------
        >>> for i, NacelleSensor_obj in enumerate(obj.Sensors_as_any):
                print(f"Element {i} is of type NacelleSensor_obj.NacelleSensorType}")

        or

        >>> for NacelleSensor_obj in obj.Sensors_as_any:
                # process object
        """
        for val in self.Sensors:
            if isinstance(val, NacelleSensorInsert) or val.is_insert == True:
                raise TypeError(f"Expected element value to be an in-line object, but it is currently in the '$insert' state.")
            yield val


    def Sensors_element_as_NacelleAccelerometer(self, index: int) -> NacelleAccelerometer:
        """
        Retrieves an object from the Sensors array field, ensuring it is a NacelleAccelerometer.        

        Useful when using type-checking development tools, as it provides a concise way to retrieve a value and assign a variable with the correct type.

        Parameters
        ----------
        index: int
            The index of the element to retrieve.

        Returns
        -------
        NacelleAccelerometer
            A model object at the specified index, guaranteed to be a NacelleAccelerometer.

        Raises
        ------
        IndexError
            If the list index is out of range

        TypeError
            If the value at the specified index is not a NacelleAccelerometer.

        Examples
        --------
        Gets a reference to the 2nd element, typed to a NacelleAccelerometer, that is specified in-line:
        >>> entry_obj = obj.Sensors_element_as_NacelleAccelerometer(2)
        """
        return self.Sensors_element_as(index, NacelleAccelerometer)


    def Sensors_element_as_NacelleInclinometer(self, index: int) -> NacelleInclinometer:
        """
        Retrieves an object from the Sensors array field, ensuring it is a NacelleInclinometer.        

        Useful when using type-checking development tools, as it provides a concise way to retrieve a value and assign a variable with the correct type.

        Parameters
        ----------
        index: int
            The index of the element to retrieve.

        Returns
        -------
        NacelleInclinometer
            A model object at the specified index, guaranteed to be a NacelleInclinometer.

        Raises
        ------
        IndexError
            If the list index is out of range

        TypeError
            If the value at the specified index is not a NacelleInclinometer.

        Examples
        --------
        Gets a reference to the 2nd element, typed to a NacelleInclinometer, that is specified in-line:
        >>> entry_obj = obj.Sensors_element_as_NacelleInclinometer(2)
        """
        return self.Sensors_element_as(index, NacelleInclinometer)


    def Sensors_element_as_inline(self, index: int) -> Union[NacelleAccelerometer, NacelleInclinometer]:
        """
        Retrieves an object from the Sensors array field; if the value is specified with a '$insert', an error is raised.

        Useful when using type-checking development tools, as it provides a concise way to retrieve a value and assign a variable with the correct type.

        Parameters
        ----------
        index: int
            The index of the element to retrieve.

        Returns
        -------
        Union[NacelleAccelerometer, NacelleInclinometer]
            A model object at the specified index, guaranteed to not be an '$insert'.

        Raises
        ------
        IndexError
            If the list index is out of range

        TypeError
            If the value at the specified index is not one of the concrete types of NacelleSensor; i.e. it is specified with an '$insert'.

        Examples
        --------
        Get a reference to the 2nd element, as any of the concrete types:
        >>> Sensors_obj = obj.Sensors_element_as_inline(2)
        """
        val = self.Sensors[index]
        if isinstance(val, NacelleSensorInsert) or val.is_insert == True:
            raise TypeError(f"Expected element value at '{index}' to be an in-line object, but it is currently in the '$insert' state.")
        return val


    def Sensors_element_as(self, index: int, element_cls: Type[TNacelleSensorOptions]) -> TNacelleSensorOptions:
        """
        Retrieves an object from the Sensors array field, ensuring it is of the specified type.
        The specified type must be one of the valid concrete types of NacelleSensor, or the '$insert' type
        (in the case the model is defined to be inserted from an external resource).

        Useful when using type-checking development tools, as it provides a concise way to access a value with the correct type.

        Parameters
        ----------
        index: int
            The index of the element to retrieve.

        element_cls: Type[Union[NacelleAccelerometer, NacelleInclinometer, NacelleSensorInsert]]
            One of the valid concrete types of NacelleSensor, or the '$insert' type
            (in the case the model is defined to be inserted from an external resource).

        Returns
        -------
        TNacelleSensorOptions
            A model object of the specified type.

        Raises
        ------
        IndexError
            If the list index is out of range

        TypeError
            If the model object for the specified index is not of the specified type.

        Examples
        --------
        Get a reference to the 2nd element when it was one of the types of NacelleSensor:
        >>> entry_obj = obj.Sensors_element_as(2, models.NacelleAccelerometer)
        >>> entry_obj = obj.Sensors_element_as(2, models.NacelleInclinometer)

        Get a reference to the value, when it was specified with a '$insert' and read in from a file:
        >>> insert_obj = obj.Sensors_element_as(2, models.NacelleSensorInsert)
        """
        val = self.Sensors[index]
        if not isinstance(val, element_cls):
            raise TypeError(f"Expected value of type '{element_cls.__name__}' at index {index} but found type '{type(val).__name__}'")
        return val


    @property
    def ShaftBrakes_as_inline(self) -> Iterable[Union[NonLinearShaftBrake, SimpleShaftBrake]]:
        """
        Retrieves the value of ShaftBrakes as an iterable of model objects; if any of the values are specified with a '$insert', an error is raised.

        Returns
        -------
        Iterable[Union[NonLinearShaftBrake, SimpleShaftBrake]]
            A list of model objects, guaranteed to not be a '$insert'.

        Raises
        ------
        TypeError
            If any of the values are not one of the concrete types of Brake; i.e. they are specified with an 'insert'.

        Examples
        --------
        >>> for i, Brake_obj in enumerate(obj.ShaftBrakes_as_any):
                print(f"Element {i} is of type Brake_obj.BrakeType}")

        or

        >>> for Brake_obj in obj.ShaftBrakes_as_any:
                # process object
        """
        for val in self.ShaftBrakes:
            if isinstance(val, BrakeInsert) or val.is_insert == True:
                raise TypeError(f"Expected element value to be an in-line object, but it is currently in the '$insert' state.")
            yield val


    def ShaftBrakes_element_as_NonLinearShaftBrake(self, index: int) -> NonLinearShaftBrake:
        """
        Retrieves an object from the ShaftBrakes array field, ensuring it is a NonLinearShaftBrake.        

        Useful when using type-checking development tools, as it provides a concise way to retrieve a value and assign a variable with the correct type.

        Parameters
        ----------
        index: int
            The index of the element to retrieve.

        Returns
        -------
        NonLinearShaftBrake
            A model object at the specified index, guaranteed to be a NonLinearShaftBrake.

        Raises
        ------
        IndexError
            If the list index is out of range

        TypeError
            If the value at the specified index is not a NonLinearShaftBrake.

        Examples
        --------
        Gets a reference to the 2nd element, typed to a NonLinearShaftBrake, that is specified in-line:
        >>> entry_obj = obj.ShaftBrakes_element_as_NonLinearShaftBrake(2)
        """
        return self.ShaftBrakes_element_as(index, NonLinearShaftBrake)


    def ShaftBrakes_element_as_SimpleShaftBrake(self, index: int) -> SimpleShaftBrake:
        """
        Retrieves an object from the ShaftBrakes array field, ensuring it is a SimpleShaftBrake.        

        Useful when using type-checking development tools, as it provides a concise way to retrieve a value and assign a variable with the correct type.

        Parameters
        ----------
        index: int
            The index of the element to retrieve.

        Returns
        -------
        SimpleShaftBrake
            A model object at the specified index, guaranteed to be a SimpleShaftBrake.

        Raises
        ------
        IndexError
            If the list index is out of range

        TypeError
            If the value at the specified index is not a SimpleShaftBrake.

        Examples
        --------
        Gets a reference to the 2nd element, typed to a SimpleShaftBrake, that is specified in-line:
        >>> entry_obj = obj.ShaftBrakes_element_as_SimpleShaftBrake(2)
        """
        return self.ShaftBrakes_element_as(index, SimpleShaftBrake)


    def ShaftBrakes_element_as_inline(self, index: int) -> Union[NonLinearShaftBrake, SimpleShaftBrake]:
        """
        Retrieves an object from the ShaftBrakes array field; if the value is specified with a '$insert', an error is raised.

        Useful when using type-checking development tools, as it provides a concise way to retrieve a value and assign a variable with the correct type.

        Parameters
        ----------
        index: int
            The index of the element to retrieve.

        Returns
        -------
        Union[NonLinearShaftBrake, SimpleShaftBrake]
            A model object at the specified index, guaranteed to not be an '$insert'.

        Raises
        ------
        IndexError
            If the list index is out of range

        TypeError
            If the value at the specified index is not one of the concrete types of Brake; i.e. it is specified with an '$insert'.

        Examples
        --------
        Get a reference to the 2nd element, as any of the concrete types:
        >>> ShaftBrakes_obj = obj.ShaftBrakes_element_as_inline(2)
        """
        val = self.ShaftBrakes[index]
        if isinstance(val, BrakeInsert) or val.is_insert == True:
            raise TypeError(f"Expected element value at '{index}' to be an in-line object, but it is currently in the '$insert' state.")
        return val


    def ShaftBrakes_element_as(self, index: int, element_cls: Type[TBrakeOptions]) -> TBrakeOptions:
        """
        Retrieves an object from the ShaftBrakes array field, ensuring it is of the specified type.
        The specified type must be one of the valid concrete types of Brake, or the '$insert' type
        (in the case the model is defined to be inserted from an external resource).

        Useful when using type-checking development tools, as it provides a concise way to access a value with the correct type.

        Parameters
        ----------
        index: int
            The index of the element to retrieve.

        element_cls: Type[Union[BrakeInsert, NonLinearShaftBrake, SimpleShaftBrake]]
            One of the valid concrete types of Brake, or the '$insert' type
            (in the case the model is defined to be inserted from an external resource).

        Returns
        -------
        TBrakeOptions
            A model object of the specified type.

        Raises
        ------
        IndexError
            If the list index is out of range

        TypeError
            If the model object for the specified index is not of the specified type.

        Examples
        --------
        Get a reference to the 2nd element when it was one of the types of Brake:
        >>> entry_obj = obj.ShaftBrakes_element_as(2, models.NonLinearShaftBrake)
        >>> entry_obj = obj.ShaftBrakes_element_as(2, models.SimpleShaftBrake)

        Get a reference to the value, when it was specified with a '$insert' and read in from a file:
        >>> insert_obj = obj.ShaftBrakes_element_as(2, models.BrakeInsert)
        """
        val = self.ShaftBrakes[index]
        if not isinstance(val, element_cls):
            raise TypeError(f"Expected value of type '{element_cls.__name__}' at index {index} but found type '{type(val).__name__}'")
        return val


    def _entity(self) -> bool:
        return True


DrivetrainAndNacelle.update_forward_refs()



# *********************************************************************#
# Bladed Python Models API                                             #
# Copyright (c) DNV Services UK Limited (c) 2025. All rights reserved. #
# MIT License (see license file)                                       #
# *********************************************************************#


# coding: utf-8

from __future__ import annotations

from abc import ABC

from datetime import date, datetime  # noqa: F401
from enum import Enum, IntEnum

import os
import re  # noqa: F401
from typing import Annotated, Any, Dict, List, Literal, Optional, Set, Type, Union, Callable, Iterable  # noqa: F401
from pathlib import Path
from typing import TypeVar
Model = TypeVar('Model', bound='BaseModel')
StrBytes = Union[str, bytes]

from pydantic import AnyUrl, BaseModel, EmailStr, Field, validator, root_validator, Extra,PrivateAttr  # noqa: F401
from pydantic import ValidationError
from pydantic.error_wrappers import ErrorWrapper
from pydantic.utils import ROOT_KEY
from json import encoder
from dnv_bladed_models.bladed_model import BladedModel
class SteadyCalculation_SteadyCalculationTypeEnum(str, Enum):
    AERODYNAMIC_INFORMATION = "AerodynamicInformation"
    BLADE_STABILITY_ANALYSIS = "BladeStabilityAnalysis"
    CAMPBELL_DIAGRAM = "CampbellDiagram"
    MODAL_ANALYSIS = "ModalAnalysis"
    MODEL_LINEARISATION = "ModelLinearisation"
    PERFORMANCE_COEFFICIENTS = "PerformanceCoefficients"
    STEADY_OPERATIONAL_LOADS = "SteadyOperationalLoads"
    STEADY_PARKED_LOADS = "SteadyParkedLoads"
    STEADY_POWER_CURVE = "SteadyPowerCurve"
    INSERT = "Insert"

from .schema_helper import SchemaHelper
from .models_impl import *


class SteadyCalculation(BladedModel, ABC):
    r"""
    The common properties of the steady calculations in Bladed.
    
    Attributes
    ----------
    SteadyCalculationType : SteadyCalculation_SteadyCalculationTypeEnum
        Defines the specific type of model in use.
    
    Notes
    -----
    
    This class is an abstraction, with the following concrete implementations:
        - AerodynamicInformationCalculation
        - BladeStabilityAnalysis
        - CampbellDiagram
        - SteadyCalculationInsert
        - ModalAnalysisCalculation
        - ModelLinearisation
        - PerformanceCoefficientsCalculation
        - SteadyOperationalLoadsCalculation
        - SteadyParkedLoadsCalculation
        - SteadyPowerCurveCalculation
    
    """
    SteadyCalculationType: SteadyCalculation_SteadyCalculationTypeEnum = Field(alias="SteadyCalculationType", default=None)

    _type_info = TypeInfo(
        set([]),
        set([]),
        set([]),
        None).merge(BladedModel._type_info)


SteadyCalculation.update_forward_refs()



# *********************************************************************#
# Bladed Python Models API                                             #
# Copyright (c) DNV Services UK Limited (c) 2025. All rights reserved. #
# MIT License (see license file)                                       #
# *********************************************************************#


# coding: utf-8

from __future__ import annotations

from datetime import date, datetime  # noqa: F401
from enum import Enum, IntEnum

import os
import re  # noqa: F401
from typing import Annotated, Any, Dict, List, Literal, Optional, Set, Type, Union, Callable, Iterable  # noqa: F401
from pathlib import Path
from typing import TypeVar
Model = TypeVar('Model', bound='BaseModel')
StrBytes = Union[str, bytes]

from pydantic import AnyUrl, BaseModel, EmailStr, Field, validator, root_validator, Extra,PrivateAttr  # noqa: F401
from pydantic import ValidationError
from pydantic.error_wrappers import ErrorWrapper
from pydantic.utils import ROOT_KEY
from json import encoder
from dnv_bladed_models.preset_wind_direction_shear_transient import PresetWindDirectionShearTransient
from dnv_bladed_models.preset_wind_horizontal_shear_transient import PresetWindHorizontalShearTransient
from dnv_bladed_models.preset_wind_mean_speed_transient import PresetWindMeanSpeedTransient
from dnv_bladed_models.preset_wind_vertical_shear_transient import PresetWindVerticalShearTransient
from dnv_bladed_models.time_domain_wind import TimeDomainWind
from dnv_bladed_models.wind_direction_shear_time_history import WindDirectionShearTimeHistory
from dnv_bladed_models.wind_direction_shear_variation import WindDirectionShearVariation
from dnv_bladed_models.wind_direction_shear_variation_insert import WindDirectionShearVariationInsert
from dnv_bladed_models.wind_horizontal_shear_time_history import WindHorizontalShearTimeHistory
from dnv_bladed_models.wind_horizontal_shear_variation import WindHorizontalShearVariation
from dnv_bladed_models.wind_horizontal_shear_variation_insert import WindHorizontalShearVariationInsert
from dnv_bladed_models.wind_mean_speed_time_history import WindMeanSpeedTimeHistory
from dnv_bladed_models.wind_mean_speed_variation import WindMeanSpeedVariation
from dnv_bladed_models.wind_mean_speed_variation_insert import WindMeanSpeedVariationInsert
from dnv_bladed_models.wind_vertical_shear_time_history import WindVerticalShearTimeHistory
from dnv_bladed_models.wind_vertical_shear_variation import WindVerticalShearVariation
from dnv_bladed_models.wind_vertical_shear_variation_insert import WindVerticalShearVariationInsert

from .schema_helper import SchemaHelper
from .models_impl import *

TWindMeanSpeedVariationOptions = TypeVar('TWindMeanSpeedVariationOptions', WindMeanSpeedVariationInsert, PresetWindMeanSpeedTransient, WindMeanSpeedTimeHistory, WindMeanSpeedVariation, )
TWindVerticalShearVariationOptions = TypeVar('TWindVerticalShearVariationOptions', WindVerticalShearVariationInsert, PresetWindVerticalShearTransient, WindVerticalShearTimeHistory, WindVerticalShearVariation, )
TWindHorizontalShearVariationOptions = TypeVar('TWindHorizontalShearVariationOptions', WindHorizontalShearVariationInsert, PresetWindHorizontalShearTransient, WindHorizontalShearTimeHistory, WindHorizontalShearVariation, )
TWindDirectionShearVariationOptions = TypeVar('TWindDirectionShearVariationOptions', WindDirectionShearVariationInsert, PresetWindDirectionShearTransient, WindDirectionShearTimeHistory, WindDirectionShearVariation, )

class LaminarFlowWind(TimeDomainWind):
    r"""
    The definition of a wind field that varies throughout a time domain simulation, but does not have turbulence.
    
    Attributes
    ----------
    WindType : Literal['LaminarFlow'], default='LaminarFlow'
        Defines the specific type of Wind model in use.  For a `LaminarFlow` object, this must always be set to a value of `LaminarFlow`.
    
    MeanSpeed : float
        The (constant) mean wind speed for the duration of the simulation.
    
    MeanSpeedVariation : Union[WindMeanSpeedVariationInsert, PresetWindMeanSpeedTransient, WindMeanSpeedTimeHistory]
    
    VerticalShear : float, default=0
        The exponential factor governing the shape of the wind shear variation.  This is typically in the order of 0.2.
    
    VerticalShearVariation : Union[WindVerticalShearVariationInsert, PresetWindVerticalShearTransient, WindVerticalShearTimeHistory]
    
    HorizontalShear : float, default=0
        The change in wind velocity for every unit of distance to either side of the hub.  This represents a linear horizontal wind shear, where the wind speed at the hub centre is at the free-field speed.
    
    HorizontalShearVariation : Union[WindHorizontalShearVariationInsert, PresetWindHorizontalShearTransient, WindHorizontalShearTimeHistory]
    
    DirectionShear : float, default=0
        The direction shear, otherwise known as \"wind veer\".  This models the case where the direction of the wind field varies as the height increases.
    
    DirectionShearVariation : Union[WindDirectionShearVariationInsert, PresetWindDirectionShearTransient, WindDirectionShearTimeHistory]
    
    UseGustPropagation : bool
        If true, gust propagation will be applied (where the transient properties only \"arrive\" at the turbine as the flow does).  This is only relevant to transient flows.
    
    Notes
    -----
    
    """
    WindType: Literal['LaminarFlow'] = Field(alias="WindType", default='LaminarFlow', allow_mutation=False, const=True) # type: ignore
    MeanSpeed: float = Field(alias="MeanSpeed", default=None)
    MeanSpeedVariation: Union[WindMeanSpeedVariationInsert, PresetWindMeanSpeedTransient, WindMeanSpeedTimeHistory] = Field(alias="MeanSpeedVariation", default=None, discriminator='MeanSpeedVariationType')
    VerticalShear: float = Field(alias="VerticalShear", default=None)
    VerticalShearVariation: Union[WindVerticalShearVariationInsert, PresetWindVerticalShearTransient, WindVerticalShearTimeHistory] = Field(alias="VerticalShearVariation", default=None, discriminator='VerticalShearVariationType')
    HorizontalShear: float = Field(alias="HorizontalShear", default=None)
    HorizontalShearVariation: Union[WindHorizontalShearVariationInsert, PresetWindHorizontalShearTransient, WindHorizontalShearTimeHistory] = Field(alias="HorizontalShearVariation", default=None, discriminator='HorizontalShearVariationType')
    DirectionShear: float = Field(alias="DirectionShear", default=None)
    DirectionShearVariation: Union[WindDirectionShearVariationInsert, PresetWindDirectionShearTransient, WindDirectionShearTimeHistory] = Field(alias="DirectionShearVariation", default=None, discriminator='DirectionShearVariationType')
    UseGustPropagation: bool = Field(alias="UseGustPropagation", default=None)

    _relative_schema_path = 'TimeDomainSimulation/Environment/Wind/LaminarFlowWind.json'
    _type_info = TypeInfo(
        set([('MeanSpeedVariation', 'MeanSpeedVariationType'),('VerticalShearVariation', 'VerticalShearVariationType'),('HorizontalShearVariation', 'HorizontalShearVariationType'),('DirectionShearVariation', 'DirectionShearVariationType'),]),
        set([]),
        set([]),
        'WindType').merge(TimeDomainWind._type_info)


    class Config:
        extra = Extra.forbid
        validate_assignment = True
        allow_population_by_field_name = True
        pass

    @property
    def MeanSpeedVariation_as_PresetWindMeanSpeedTransient(self) -> PresetWindMeanSpeedTransient:
        """
        Retrieves the value of MeanSpeedVariation guaranteeing it is a PresetWindMeanSpeedTransient; if it is not, an error is raised.

        Useful when using type-checking development tools, as it provides a concise way to retrieve a value and assign a variable with the correct type.

        Returns
        -------
        PresetWindMeanSpeedTransient
            A model object, guaranteed to be a PresetWindMeanSpeedTransient.

        Raises
        ------
        TypeError
            If the value is not a PresetWindMeanSpeedTransient.
        """
        return self.MeanSpeedVariation_as(PresetWindMeanSpeedTransient)


    @property
    def MeanSpeedVariation_as_WindMeanSpeedTimeHistory(self) -> WindMeanSpeedTimeHistory:
        """
        Retrieves the value of MeanSpeedVariation guaranteeing it is a WindMeanSpeedTimeHistory; if it is not, an error is raised.

        Useful when using type-checking development tools, as it provides a concise way to retrieve a value and assign a variable with the correct type.

        Returns
        -------
        WindMeanSpeedTimeHistory
            A model object, guaranteed to be a WindMeanSpeedTimeHistory.

        Raises
        ------
        TypeError
            If the value is not a WindMeanSpeedTimeHistory.
        """
        return self.MeanSpeedVariation_as(WindMeanSpeedTimeHistory)


    @property
    def MeanSpeedVariation_as_inline(self) -> Union[PresetWindMeanSpeedTransient, WindMeanSpeedTimeHistory]:
        """
        Retrieves the value of MeanSpeedVariation as a model object; if the value is specified with a '$insert', an error is raised.

        Returns
        -------
        Union[PresetWindMeanSpeedTransient, WindMeanSpeedTimeHistory]
            A model object at the specified index, guaranteed to not be a '$insert'.

        Raises
        ------
        TypeError
            If the value is not one of the concrete types of WindMeanSpeedVariation; i.e. it is specified with a '$insert'.
        """
        if isinstance(self.MeanSpeedVariation, WindMeanSpeedVariationInsert) or self.MeanSpeedVariation.is_insert:
            raise TypeError(f"Expected MeanSpeedVariation value to be an in-line object, but it is currently in the '$insert' state.")
        return self.MeanSpeedVariation


    def MeanSpeedVariation_as(self, cls: Type[TWindMeanSpeedVariationOptions])-> TWindMeanSpeedVariationOptions:
        """
        Retrieves the value of MeanSpeedVariation, ensuring it is of the specified type.
        The specified type must be one of the valid concrete types of WindMeanSpeedVariation, or the '$insert' type
        (in the case the model is defined to be inserted from an external resource).

        Useful when using type-checking development tools, as it provides a concise way to access a value with the correct type.

        Parameters
        ----------
        cls: Type[Union[WindMeanSpeedVariationInsert, PresetWindMeanSpeedTransient, WindMeanSpeedTimeHistory]]
            One of the valid concrete types of WindMeanSpeedVariation, or the '$insert' type
            (in the case the model is defined to be inserted from an external resource).

        Returns
        -------
        TWindMeanSpeedVariationOptions
            A model object of the specified type.

        Raises
        ------
        TypeError
            If the value is not of the specified type.

        Examples
        --------
        Get a reference to the value when it is one of the types of WindMeanSpeedVariation:
        >>> val_obj = model_obj.MeanSpeedVariation_as(models.PresetWindMeanSpeedTransient)
        >>> val_obj = model_obj.MeanSpeedVariation_as(models.WindMeanSpeedTimeHistory)

        Get a reference to the value, when it was specified with a '$insert' and read in from a file:
        >>> insert_obj = model_obj.MeanSpeedVariation_as(models.WindMeanSpeedVariationInsert)
        """
        if not isinstance(self.MeanSpeedVariation, cls):
            raise TypeError(f"Expected MeanSpeedVariation of type '{cls.__name__}' but was type '{type(self.MeanSpeedVariation).__name__}'")
        return self.MeanSpeedVariation


    @property
    def VerticalShearVariation_as_PresetWindVerticalShearTransient(self) -> PresetWindVerticalShearTransient:
        """
        Retrieves the value of VerticalShearVariation guaranteeing it is a PresetWindVerticalShearTransient; if it is not, an error is raised.

        Useful when using type-checking development tools, as it provides a concise way to retrieve a value and assign a variable with the correct type.

        Returns
        -------
        PresetWindVerticalShearTransient
            A model object, guaranteed to be a PresetWindVerticalShearTransient.

        Raises
        ------
        TypeError
            If the value is not a PresetWindVerticalShearTransient.
        """
        return self.VerticalShearVariation_as(PresetWindVerticalShearTransient)


    @property
    def VerticalShearVariation_as_WindVerticalShearTimeHistory(self) -> WindVerticalShearTimeHistory:
        """
        Retrieves the value of VerticalShearVariation guaranteeing it is a WindVerticalShearTimeHistory; if it is not, an error is raised.

        Useful when using type-checking development tools, as it provides a concise way to retrieve a value and assign a variable with the correct type.

        Returns
        -------
        WindVerticalShearTimeHistory
            A model object, guaranteed to be a WindVerticalShearTimeHistory.

        Raises
        ------
        TypeError
            If the value is not a WindVerticalShearTimeHistory.
        """
        return self.VerticalShearVariation_as(WindVerticalShearTimeHistory)


    @property
    def VerticalShearVariation_as_inline(self) -> Union[PresetWindVerticalShearTransient, WindVerticalShearTimeHistory]:
        """
        Retrieves the value of VerticalShearVariation as a model object; if the value is specified with a '$insert', an error is raised.

        Returns
        -------
        Union[PresetWindVerticalShearTransient, WindVerticalShearTimeHistory]
            A model object at the specified index, guaranteed to not be a '$insert'.

        Raises
        ------
        TypeError
            If the value is not one of the concrete types of WindVerticalShearVariation; i.e. it is specified with a '$insert'.
        """
        if isinstance(self.VerticalShearVariation, WindVerticalShearVariationInsert) or self.VerticalShearVariation.is_insert:
            raise TypeError(f"Expected VerticalShearVariation value to be an in-line object, but it is currently in the '$insert' state.")
        return self.VerticalShearVariation


    def VerticalShearVariation_as(self, cls: Type[TWindVerticalShearVariationOptions])-> TWindVerticalShearVariationOptions:
        """
        Retrieves the value of VerticalShearVariation, ensuring it is of the specified type.
        The specified type must be one of the valid concrete types of WindVerticalShearVariation, or the '$insert' type
        (in the case the model is defined to be inserted from an external resource).

        Useful when using type-checking development tools, as it provides a concise way to access a value with the correct type.

        Parameters
        ----------
        cls: Type[Union[WindVerticalShearVariationInsert, PresetWindVerticalShearTransient, WindVerticalShearTimeHistory]]
            One of the valid concrete types of WindVerticalShearVariation, or the '$insert' type
            (in the case the model is defined to be inserted from an external resource).

        Returns
        -------
        TWindVerticalShearVariationOptions
            A model object of the specified type.

        Raises
        ------
        TypeError
            If the value is not of the specified type.

        Examples
        --------
        Get a reference to the value when it is one of the types of WindVerticalShearVariation:
        >>> val_obj = model_obj.VerticalShearVariation_as(models.PresetWindVerticalShearTransient)
        >>> val_obj = model_obj.VerticalShearVariation_as(models.WindVerticalShearTimeHistory)

        Get a reference to the value, when it was specified with a '$insert' and read in from a file:
        >>> insert_obj = model_obj.VerticalShearVariation_as(models.WindVerticalShearVariationInsert)
        """
        if not isinstance(self.VerticalShearVariation, cls):
            raise TypeError(f"Expected VerticalShearVariation of type '{cls.__name__}' but was type '{type(self.VerticalShearVariation).__name__}'")
        return self.VerticalShearVariation


    @property
    def HorizontalShearVariation_as_PresetWindHorizontalShearTransient(self) -> PresetWindHorizontalShearTransient:
        """
        Retrieves the value of HorizontalShearVariation guaranteeing it is a PresetWindHorizontalShearTransient; if it is not, an error is raised.

        Useful when using type-checking development tools, as it provides a concise way to retrieve a value and assign a variable with the correct type.

        Returns
        -------
        PresetWindHorizontalShearTransient
            A model object, guaranteed to be a PresetWindHorizontalShearTransient.

        Raises
        ------
        TypeError
            If the value is not a PresetWindHorizontalShearTransient.
        """
        return self.HorizontalShearVariation_as(PresetWindHorizontalShearTransient)


    @property
    def HorizontalShearVariation_as_WindHorizontalShearTimeHistory(self) -> WindHorizontalShearTimeHistory:
        """
        Retrieves the value of HorizontalShearVariation guaranteeing it is a WindHorizontalShearTimeHistory; if it is not, an error is raised.

        Useful when using type-checking development tools, as it provides a concise way to retrieve a value and assign a variable with the correct type.

        Returns
        -------
        WindHorizontalShearTimeHistory
            A model object, guaranteed to be a WindHorizontalShearTimeHistory.

        Raises
        ------
        TypeError
            If the value is not a WindHorizontalShearTimeHistory.
        """
        return self.HorizontalShearVariation_as(WindHorizontalShearTimeHistory)


    @property
    def HorizontalShearVariation_as_inline(self) -> Union[PresetWindHorizontalShearTransient, WindHorizontalShearTimeHistory]:
        """
        Retrieves the value of HorizontalShearVariation as a model object; if the value is specified with a '$insert', an error is raised.

        Returns
        -------
        Union[PresetWindHorizontalShearTransient, WindHorizontalShearTimeHistory]
            A model object at the specified index, guaranteed to not be a '$insert'.

        Raises
        ------
        TypeError
            If the value is not one of the concrete types of WindHorizontalShearVariation; i.e. it is specified with a '$insert'.
        """
        if isinstance(self.HorizontalShearVariation, WindHorizontalShearVariationInsert) or self.HorizontalShearVariation.is_insert:
            raise TypeError(f"Expected HorizontalShearVariation value to be an in-line object, but it is currently in the '$insert' state.")
        return self.HorizontalShearVariation


    def HorizontalShearVariation_as(self, cls: Type[TWindHorizontalShearVariationOptions])-> TWindHorizontalShearVariationOptions:
        """
        Retrieves the value of HorizontalShearVariation, ensuring it is of the specified type.
        The specified type must be one of the valid concrete types of WindHorizontalShearVariation, or the '$insert' type
        (in the case the model is defined to be inserted from an external resource).

        Useful when using type-checking development tools, as it provides a concise way to access a value with the correct type.

        Parameters
        ----------
        cls: Type[Union[WindHorizontalShearVariationInsert, PresetWindHorizontalShearTransient, WindHorizontalShearTimeHistory]]
            One of the valid concrete types of WindHorizontalShearVariation, or the '$insert' type
            (in the case the model is defined to be inserted from an external resource).

        Returns
        -------
        TWindHorizontalShearVariationOptions
            A model object of the specified type.

        Raises
        ------
        TypeError
            If the value is not of the specified type.

        Examples
        --------
        Get a reference to the value when it is one of the types of WindHorizontalShearVariation:
        >>> val_obj = model_obj.HorizontalShearVariation_as(models.PresetWindHorizontalShearTransient)
        >>> val_obj = model_obj.HorizontalShearVariation_as(models.WindHorizontalShearTimeHistory)

        Get a reference to the value, when it was specified with a '$insert' and read in from a file:
        >>> insert_obj = model_obj.HorizontalShearVariation_as(models.WindHorizontalShearVariationInsert)
        """
        if not isinstance(self.HorizontalShearVariation, cls):
            raise TypeError(f"Expected HorizontalShearVariation of type '{cls.__name__}' but was type '{type(self.HorizontalShearVariation).__name__}'")
        return self.HorizontalShearVariation


    @property
    def DirectionShearVariation_as_PresetWindDirectionShearTransient(self) -> PresetWindDirectionShearTransient:
        """
        Retrieves the value of DirectionShearVariation guaranteeing it is a PresetWindDirectionShearTransient; if it is not, an error is raised.

        Useful when using type-checking development tools, as it provides a concise way to retrieve a value and assign a variable with the correct type.

        Returns
        -------
        PresetWindDirectionShearTransient
            A model object, guaranteed to be a PresetWindDirectionShearTransient.

        Raises
        ------
        TypeError
            If the value is not a PresetWindDirectionShearTransient.
        """
        return self.DirectionShearVariation_as(PresetWindDirectionShearTransient)


    @property
    def DirectionShearVariation_as_WindDirectionShearTimeHistory(self) -> WindDirectionShearTimeHistory:
        """
        Retrieves the value of DirectionShearVariation guaranteeing it is a WindDirectionShearTimeHistory; if it is not, an error is raised.

        Useful when using type-checking development tools, as it provides a concise way to retrieve a value and assign a variable with the correct type.

        Returns
        -------
        WindDirectionShearTimeHistory
            A model object, guaranteed to be a WindDirectionShearTimeHistory.

        Raises
        ------
        TypeError
            If the value is not a WindDirectionShearTimeHistory.
        """
        return self.DirectionShearVariation_as(WindDirectionShearTimeHistory)


    @property
    def DirectionShearVariation_as_inline(self) -> Union[PresetWindDirectionShearTransient, WindDirectionShearTimeHistory]:
        """
        Retrieves the value of DirectionShearVariation as a model object; if the value is specified with a '$insert', an error is raised.

        Returns
        -------
        Union[PresetWindDirectionShearTransient, WindDirectionShearTimeHistory]
            A model object at the specified index, guaranteed to not be a '$insert'.

        Raises
        ------
        TypeError
            If the value is not one of the concrete types of WindDirectionShearVariation; i.e. it is specified with a '$insert'.
        """
        if isinstance(self.DirectionShearVariation, WindDirectionShearVariationInsert) or self.DirectionShearVariation.is_insert:
            raise TypeError(f"Expected DirectionShearVariation value to be an in-line object, but it is currently in the '$insert' state.")
        return self.DirectionShearVariation


    def DirectionShearVariation_as(self, cls: Type[TWindDirectionShearVariationOptions])-> TWindDirectionShearVariationOptions:
        """
        Retrieves the value of DirectionShearVariation, ensuring it is of the specified type.
        The specified type must be one of the valid concrete types of WindDirectionShearVariation, or the '$insert' type
        (in the case the model is defined to be inserted from an external resource).

        Useful when using type-checking development tools, as it provides a concise way to access a value with the correct type.

        Parameters
        ----------
        cls: Type[Union[WindDirectionShearVariationInsert, PresetWindDirectionShearTransient, WindDirectionShearTimeHistory]]
            One of the valid concrete types of WindDirectionShearVariation, or the '$insert' type
            (in the case the model is defined to be inserted from an external resource).

        Returns
        -------
        TWindDirectionShearVariationOptions
            A model object of the specified type.

        Raises
        ------
        TypeError
            If the value is not of the specified type.

        Examples
        --------
        Get a reference to the value when it is one of the types of WindDirectionShearVariation:
        >>> val_obj = model_obj.DirectionShearVariation_as(models.PresetWindDirectionShearTransient)
        >>> val_obj = model_obj.DirectionShearVariation_as(models.WindDirectionShearTimeHistory)

        Get a reference to the value, when it was specified with a '$insert' and read in from a file:
        >>> insert_obj = model_obj.DirectionShearVariation_as(models.WindDirectionShearVariationInsert)
        """
        if not isinstance(self.DirectionShearVariation, cls):
            raise TypeError(f"Expected DirectionShearVariation of type '{cls.__name__}' but was type '{type(self.DirectionShearVariation).__name__}'")
        return self.DirectionShearVariation


    def _entity(self) -> bool:
        return True


LaminarFlowWind.update_forward_refs()



# *********************************************************************#
# Bladed Python Models API                                             #
# Copyright (c) DNV Services UK Limited (c) 2025. All rights reserved. #
# MIT License (see license file)                                       #
# *********************************************************************#


# coding: utf-8

from __future__ import annotations

from datetime import date, datetime  # noqa: F401
from enum import Enum, IntEnum

import os
import re  # noqa: F401
from typing import Annotated, Any, Dict, List, Literal, Optional, Set, Type, Union, Callable, Iterable  # noqa: F401
from pathlib import Path
from typing import TypeVar
Model = TypeVar('Model', bound='BaseModel')
StrBytes = Union[str, bytes]

from pydantic import AnyUrl, BaseModel, EmailStr, Field, validator, root_validator, Extra,PrivateAttr  # noqa: F401
from pydantic import ValidationError
from pydantic.error_wrappers import ErrorWrapper
from pydantic.utils import ROOT_KEY
from json import encoder
from dnv_bladed_models.bladed_model import BladedModel
from dnv_bladed_models.pitch_position_vs_minimum_and_maximum_moment import PitchPositionVsMinimumAndMaximumMoment
from dnv_bladed_models.pitch_rate_vs_minimum_and_maximum_moment import PitchRateVsMinimumAndMaximumMoment
from dnv_bladed_models.pitch_torque_minimum_maximum import PitchTorqueMinimumMaximum

from .schema_helper import SchemaHelper
from .models_impl import *


class PitchTorqueLimits(BladedModel):
    r"""
    The limits on the torque produced by the rotary drive.
    
    Attributes
    ----------
    Limits : PitchTorqueMinimumMaximum
    
    PositionVsLimits : List[PitchPositionVsMinimumAndMaximumMoment]
        The limits on the torque of the rotary drive as they vary with position.
    
    RateVsLimits : List[PitchRateVsMinimumAndMaximumMoment]
        The limits on the torque of the rotary drive as they vary with the angular velocity.
    
    Notes
    -----
    
    """
    Limits: PitchTorqueMinimumMaximum = Field(alias="Limits", default=None)
    PositionVsLimits: List[PitchPositionVsMinimumAndMaximumMoment] = Field(alias="PositionVsLimits", default=list())
    RateVsLimits: List[PitchRateVsMinimumAndMaximumMoment] = Field(alias="RateVsLimits", default=list())

    _relative_schema_path = 'Components/PitchSystem/PitchActuator/PitchTorqueLimits/PitchTorqueLimits.json'
    _type_info = TypeInfo(
        set([]),
        set([]),
        set(['PositionVsLimits','RateVsLimits',]),
        None).merge(BladedModel._type_info)


    class Config:
        extra = Extra.forbid
        validate_assignment = True
        allow_population_by_field_name = True
        pass

    def _entity(self) -> bool:
        return True


PitchTorqueLimits.update_forward_refs()



# *********************************************************************#
# Bladed Python Models API                                             #
# Copyright (c) DNV Services UK Limited (c) 2025. All rights reserved. #
# MIT License (see license file)                                       #
# *********************************************************************#


# coding: utf-8

from __future__ import annotations

from abc import ABC

from datetime import date, datetime  # noqa: F401
from enum import Enum, IntEnum

import os
import re  # noqa: F401
from typing import Annotated, Any, Dict, List, Literal, Optional, Set, Type, Union, Callable, Iterable  # noqa: F401
from pathlib import Path
from typing import TypeVar
Model = TypeVar('Model', bound='BaseModel')
StrBytes = Union[str, bytes]

from pydantic import AnyUrl, BaseModel, EmailStr, Field, validator, root_validator, Extra,PrivateAttr  # noqa: F401
from pydantic import ValidationError
from pydantic.error_wrappers import ErrorWrapper
from pydantic.utils import ROOT_KEY
from json import encoder
from dnv_bladed_models.bladed_model import BladedModel
from dnv_bladed_models.first_order_transducer_response import FirstOrderTransducerResponse
from dnv_bladed_models.instantaneous_transducer_response import InstantaneousTransducerResponse
from dnv_bladed_models.passive_transducer_response import PassiveTransducerResponse
from dnv_bladed_models.position_proportional_integral_deriviative import PositionProportionalIntegralDeriviative
from dnv_bladed_models.rate_proportional_integral_deriviative import RateProportionalIntegralDeriviative
from dnv_bladed_models.second_order_transducer_response import SecondOrderTransducerResponse
from dnv_bladed_models.transducer_behaviour import TransducerBehaviour
from dnv_bladed_models.transducer_behaviour_insert import TransducerBehaviourInsert
from dnv_bladed_models.use_setpoint_trajectory_planning import UseSetpointTrajectoryPlanning
class SignalProperties_SignalQualityEnum(str, Enum):
    RAW = "Raw"
    TRANSDUCER = "Transducer"
    MEASURED = "Measured"
class SignalProperties_SignalNoiseEnum(str, Enum):
    NONE = "None"
    UNIFORM = "Uniform"
    GAUSSIAN = "Gaussian"

from .schema_helper import SchemaHelper
from .models_impl import *

TTransducerBehaviourOptions = TypeVar('TTransducerBehaviourOptions', FirstOrderTransducerResponse, TransducerBehaviourInsert, InstantaneousTransducerResponse, PassiveTransducerResponse, PositionProportionalIntegralDeriviative, RateProportionalIntegralDeriviative, SecondOrderTransducerResponse, UseSetpointTrajectoryPlanning, TransducerBehaviour, )

class SignalProperties(BladedModel, ABC):
    r"""
    
    
    Not supported yet.
    
    Attributes
    ----------
    SignalQuality : SignalProperties_SignalQualityEnum, default='Raw', Not supported yet
        The representation of the signal quality - whether it has transducer lag and signal noise.
    
    SignalNoise : SignalProperties_SignalNoiseEnum, default='None', Not supported yet
        The type of noise on the measured signal.
    
    NoiseMagnitude : float, default=0, Not supported yet
        The magnitude of the signal noise.
    
    SamplingPeriod : float, default=0, Not supported yet
        The time step at which the input (continuous) signal is discretised at.
    
    DiscretisationStep : float, default=0, Not supported yet
        The intervals at which values can be represented.
    
    Transducer : Union[FirstOrderTransducerResponse, TransducerBehaviourInsert, InstantaneousTransducerResponse, PassiveTransducerResponse, PositionProportionalIntegralDeriviative, RateProportionalIntegralDeriviative, SecondOrderTransducerResponse, UseSetpointTrajectoryPlanning], Not supported yet
    
    Notes
    -----
    This model is not supported yet by the Bladed calculation engine.
    """
    SignalQuality: SignalProperties_SignalQualityEnum = Field(alias="SignalQuality", default=None) # Not supported yet
    SignalNoise: SignalProperties_SignalNoiseEnum = Field(alias="SignalNoise", default=None) # Not supported yet
    NoiseMagnitude: float = Field(alias="NoiseMagnitude", default=None) # Not supported yet
    SamplingPeriod: float = Field(alias="SamplingPeriod", default=None) # Not supported yet
    DiscretisationStep: float = Field(alias="DiscretisationStep", default=None) # Not supported yet
    Transducer: Union[FirstOrderTransducerResponse, TransducerBehaviourInsert, InstantaneousTransducerResponse, PassiveTransducerResponse, PositionProportionalIntegralDeriviative, RateProportionalIntegralDeriviative, SecondOrderTransducerResponse, UseSetpointTrajectoryPlanning] = Field(alias="Transducer", default=None, discriminator='TransducerBehaviourType') # Not supported yet

    _type_info = TypeInfo(
        set([('Transducer', 'TransducerBehaviourType'),]),
        set([]),
        set([]),
        None).merge(BladedModel._type_info)


    @property
    def Transducer_as_FirstOrderTransducerResponse(self) -> FirstOrderTransducerResponse:
        """
        Retrieves the value of Transducer guaranteeing it is a FirstOrderTransducerResponse; if it is not, an error is raised.

        Useful when using type-checking development tools, as it provides a concise way to retrieve a value and assign a variable with the correct type.

        Returns
        -------
        FirstOrderTransducerResponse
            A model object, guaranteed to be a FirstOrderTransducerResponse.

        Raises
        ------
        TypeError
            If the value is not a FirstOrderTransducerResponse.
        """
        return self.Transducer_as(FirstOrderTransducerResponse)


    @property
    def Transducer_as_InstantaneousTransducerResponse(self) -> InstantaneousTransducerResponse:
        """
        Retrieves the value of Transducer guaranteeing it is a InstantaneousTransducerResponse; if it is not, an error is raised.

        Useful when using type-checking development tools, as it provides a concise way to retrieve a value and assign a variable with the correct type.

        Returns
        -------
        InstantaneousTransducerResponse
            A model object, guaranteed to be a InstantaneousTransducerResponse.

        Raises
        ------
        TypeError
            If the value is not a InstantaneousTransducerResponse.
        """
        return self.Transducer_as(InstantaneousTransducerResponse)


    @property
    def Transducer_as_PassiveTransducerResponse(self) -> PassiveTransducerResponse:
        """
        Retrieves the value of Transducer guaranteeing it is a PassiveTransducerResponse; if it is not, an error is raised.

        Useful when using type-checking development tools, as it provides a concise way to retrieve a value and assign a variable with the correct type.

        Returns
        -------
        PassiveTransducerResponse
            A model object, guaranteed to be a PassiveTransducerResponse.

        Raises
        ------
        TypeError
            If the value is not a PassiveTransducerResponse.
        """
        return self.Transducer_as(PassiveTransducerResponse)


    @property
    def Transducer_as_PositionProportionalIntegralDeriviative(self) -> PositionProportionalIntegralDeriviative:
        """
        Retrieves the value of Transducer guaranteeing it is a PositionProportionalIntegralDeriviative; if it is not, an error is raised.

        Useful when using type-checking development tools, as it provides a concise way to retrieve a value and assign a variable with the correct type.

        Returns
        -------
        PositionProportionalIntegralDeriviative
            A model object, guaranteed to be a PositionProportionalIntegralDeriviative.

        Raises
        ------
        TypeError
            If the value is not a PositionProportionalIntegralDeriviative.
        """
        return self.Transducer_as(PositionProportionalIntegralDeriviative)


    @property
    def Transducer_as_RateProportionalIntegralDeriviative(self) -> RateProportionalIntegralDeriviative:
        """
        Retrieves the value of Transducer guaranteeing it is a RateProportionalIntegralDeriviative; if it is not, an error is raised.

        Useful when using type-checking development tools, as it provides a concise way to retrieve a value and assign a variable with the correct type.

        Returns
        -------
        RateProportionalIntegralDeriviative
            A model object, guaranteed to be a RateProportionalIntegralDeriviative.

        Raises
        ------
        TypeError
            If the value is not a RateProportionalIntegralDeriviative.
        """
        return self.Transducer_as(RateProportionalIntegralDeriviative)


    @property
    def Transducer_as_SecondOrderTransducerResponse(self) -> SecondOrderTransducerResponse:
        """
        Retrieves the value of Transducer guaranteeing it is a SecondOrderTransducerResponse; if it is not, an error is raised.

        Useful when using type-checking development tools, as it provides a concise way to retrieve a value and assign a variable with the correct type.

        Returns
        -------
        SecondOrderTransducerResponse
            A model object, guaranteed to be a SecondOrderTransducerResponse.

        Raises
        ------
        TypeError
            If the value is not a SecondOrderTransducerResponse.
        """
        return self.Transducer_as(SecondOrderTransducerResponse)


    @property
    def Transducer_as_UseSetpointTrajectoryPlanning(self) -> UseSetpointTrajectoryPlanning:
        """
        Retrieves the value of Transducer guaranteeing it is a UseSetpointTrajectoryPlanning; if it is not, an error is raised.

        Useful when using type-checking development tools, as it provides a concise way to retrieve a value and assign a variable with the correct type.

        Returns
        -------
        UseSetpointTrajectoryPlanning
            A model object, guaranteed to be a UseSetpointTrajectoryPlanning.

        Raises
        ------
        TypeError
            If the value is not a UseSetpointTrajectoryPlanning.
        """
        return self.Transducer_as(UseSetpointTrajectoryPlanning)


    @property
    def Transducer_as_inline(self) -> Union[FirstOrderTransducerResponse, InstantaneousTransducerResponse, PassiveTransducerResponse, PositionProportionalIntegralDeriviative, RateProportionalIntegralDeriviative, SecondOrderTransducerResponse, UseSetpointTrajectoryPlanning]:
        """
        Retrieves the value of Transducer as a model object; if the value is specified with a '$insert', an error is raised.

        Returns
        -------
        Union[FirstOrderTransducerResponse, InstantaneousTransducerResponse, PassiveTransducerResponse, PositionProportionalIntegralDeriviative, RateProportionalIntegralDeriviative, SecondOrderTransducerResponse, UseSetpointTrajectoryPlanning]
            A model object at the specified index, guaranteed to not be a '$insert'.

        Raises
        ------
        TypeError
            If the value is not one of the concrete types of TransducerBehaviour; i.e. it is specified with a '$insert'.
        """
        if isinstance(self.Transducer, TransducerBehaviourInsert) or self.Transducer.is_insert:
            raise TypeError(f"Expected Transducer value to be an in-line object, but it is currently in the '$insert' state.")
        return self.Transducer


    def Transducer_as(self, cls: Type[TTransducerBehaviourOptions])-> TTransducerBehaviourOptions:
        """
        Retrieves the value of Transducer, ensuring it is of the specified type.
        The specified type must be one of the valid concrete types of TransducerBehaviour, or the '$insert' type
        (in the case the model is defined to be inserted from an external resource).

        Useful when using type-checking development tools, as it provides a concise way to access a value with the correct type.

        Parameters
        ----------
        cls: Type[Union[FirstOrderTransducerResponse, TransducerBehaviourInsert, InstantaneousTransducerResponse, PassiveTransducerResponse, PositionProportionalIntegralDeriviative, RateProportionalIntegralDeriviative, SecondOrderTransducerResponse, UseSetpointTrajectoryPlanning]]
            One of the valid concrete types of TransducerBehaviour, or the '$insert' type
            (in the case the model is defined to be inserted from an external resource).

        Returns
        -------
        TTransducerBehaviourOptions
            A model object of the specified type.

        Raises
        ------
        TypeError
            If the value is not of the specified type.

        Examples
        --------
        Get a reference to the value when it is one of the types of TransducerBehaviour:
        >>> val_obj = model_obj.Transducer_as(models.FirstOrderTransducerResponse)
        >>> val_obj = model_obj.Transducer_as(models.InstantaneousTransducerResponse)
        >>> val_obj = model_obj.Transducer_as(models.PassiveTransducerResponse)
        >>> val_obj = model_obj.Transducer_as(models.PositionProportionalIntegralDeriviative)
        >>> val_obj = model_obj.Transducer_as(models.RateProportionalIntegralDeriviative)
        >>> val_obj = model_obj.Transducer_as(models.SecondOrderTransducerResponse)
        >>> val_obj = model_obj.Transducer_as(models.UseSetpointTrajectoryPlanning)

        Get a reference to the value, when it was specified with a '$insert' and read in from a file:
        >>> insert_obj = model_obj.Transducer_as(models.TransducerBehaviourInsert)
        """
        if not isinstance(self.Transducer, cls):
            raise TypeError(f"Expected Transducer of type '{cls.__name__}' but was type '{type(self.Transducer).__name__}'")
        return self.Transducer


SignalProperties.update_forward_refs()



# *********************************************************************#
# Bladed Python Models API                                             #
# Copyright (c) DNV Services UK Limited (c) 2025. All rights reserved. #
# MIT License (see license file)                                       #
# *********************************************************************#


# coding: utf-8

from __future__ import annotations

from abc import ABC

from datetime import date, datetime  # noqa: F401
from enum import Enum, IntEnum

import os
import re  # noqa: F401
from typing import Annotated, Any, Dict, List, Literal, Optional, Set, Type, Union, Callable, Iterable  # noqa: F401
from pathlib import Path
from typing import TypeVar
Model = TypeVar('Model', bound='BaseModel')
StrBytes = Union[str, bytes]

from pydantic import AnyUrl, BaseModel, EmailStr, Field, validator, root_validator, Extra,PrivateAttr  # noqa: F401
from pydantic import ValidationError
from pydantic.error_wrappers import ErrorWrapper
from pydantic.utils import ROOT_KEY
from json import encoder
from dnv_bladed_models.blade_stability_perturbations import BladeStabilityPerturbations
from dnv_bladed_models.custom_linear_model_dll import CustomLinearModelDll
from dnv_bladed_models.linearisation_calculation import LinearisationCalculation

from .schema_helper import SchemaHelper
from .models_impl import *


class BladeStabilityAnalysisCalculation(LinearisationCalculation, ABC):
    r"""
    
    
    Not supported yet.
    
    Attributes
    ----------
    AzimuthAngle : float, default=0, Not supported yet
        The fixed azimuth angle of the rotor (zero azimuth indicates blade 1 pointing upwards).
    
    MaximumPlotFrequency : float, default=125.663706144, Not supported yet
        The maximum frequency for modes to be shown in the Campbell diagram plot. The default value is 20 * 2 * 3.14159265358979323
    
    CustomLinearModelDll : CustomLinearModelDll, Not supported yet
    
    Perturbations : BladeStabilityPerturbations, Not supported yet
    
    Notes
    -----
    This model is not supported yet by the Bladed calculation engine.
    """
    AzimuthAngle: float = Field(alias="AzimuthAngle", default=None) # Not supported yet
    MaximumPlotFrequency: float = Field(alias="MaximumPlotFrequency", default=None) # Not supported yet
    CustomLinearModelDll: CustomLinearModelDll = Field(alias="CustomLinearModelDll", default=None) # Not supported yet
    Perturbations: BladeStabilityPerturbations = Field(alias="Perturbations", default=None) # Not supported yet

    _type_info = TypeInfo(
        set([]),
        set([]),
        set([]),
        None).merge(LinearisationCalculation._type_info)


BladeStabilityAnalysisCalculation.update_forward_refs()



# *********************************************************************#
# Bladed Python Models API                                             #
# Copyright (c) DNV Services UK Limited (c) 2025. All rights reserved. #
# MIT License (see license file)                                       #
# *********************************************************************#


# coding: utf-8

from __future__ import annotations

from datetime import date, datetime  # noqa: F401
from enum import Enum, IntEnum

import os
import re  # noqa: F401
from typing import Annotated, Any, Dict, List, Literal, Optional, Set, Type, Union, Callable, Iterable  # noqa: F401
from pathlib import Path
from typing import TypeVar
Model = TypeVar('Model', bound='BaseModel')
StrBytes = Union[str, bytes]

from pydantic import AnyUrl, BaseModel, EmailStr, Field, validator, root_validator, Extra,PrivateAttr  # noqa: F401
from pydantic import ValidationError
from pydantic.error_wrappers import ErrorWrapper
from pydantic.utils import ROOT_KEY
from json import encoder
from dnv_bladed_models.fixed_step_integrator import FixedStepIntegrator

from .schema_helper import SchemaHelper
from .models_impl import *


class ExplicitNewmarkBetaFixedStep(FixedStepIntegrator):
    r"""
    Settings for the Explicit Newmark Beta Fixed Step integrator.
    
    Attributes
    ----------
    IntegratorType : Literal['ExplicitNewmarkBetaFixedStep'], default='ExplicitNewmarkBetaFixedStep'
        Defines the specific type of Integrator model in use.  For a `ExplicitNewmarkBetaFixedStep` object, this must always be set to a value of `ExplicitNewmarkBetaFixedStep`.
    
    MaximumNumberOfIterations : int, default=1
        The maximum number of iterations for prescribed freedoms and first order states (e.g. dynamic stall & wake).  A value of 1 may sometimes inprecisely integrate first order states
    
    Beta : float, default=0.25
        The β parameter for the Newmark-β integration method.  The recommended value of 0.25 (with a γ value of 0.50) results in the constant average acceleration method that is unconditionally stable for linear systems.  A value of 0.26 (with a γ value of 0.52) results in a method that is close to the constant average acceleration method but includes a small amount of numerical damping to reduce unwanted vibrations of high-frequency modes. Note that the numerical damping increases with the step size.
    
    Gamma : float, default=0.5
        The γ parameter for the Newmark-β integration method.  The recommended value depends on the β parameter and given by the formula γ = 2.sqrt(β) - 0.5.  Values higher than 0.5 introduce positive numerical damping, whereas lower values introduce negative numerical damping.
    
    Notes
    -----
    
    """
    IntegratorType: Literal['ExplicitNewmarkBetaFixedStep'] = Field(alias="IntegratorType", default='ExplicitNewmarkBetaFixedStep', allow_mutation=False, const=True) # type: ignore
    MaximumNumberOfIterations: int = Field(alias="MaximumNumberOfIterations", default=None)
    Beta: float = Field(alias="Beta", default=None)
    Gamma: float = Field(alias="Gamma", default=None)

    _relative_schema_path = 'Settings/SolverSettings/Integrator/ExplicitNewmarkBetaFixedStep.json'
    _type_info = TypeInfo(
        set([]),
        set([]),
        set([]),
        'IntegratorType').merge(FixedStepIntegrator._type_info)


    class Config:
        extra = Extra.forbid
        validate_assignment = True
        allow_population_by_field_name = True
        pass

    def _entity(self) -> bool:
        return True


ExplicitNewmarkBetaFixedStep.update_forward_refs()



# *********************************************************************#
# Bladed Python Models API                                             #
# Copyright (c) DNV Services UK Limited (c) 2025. All rights reserved. #
# MIT License (see license file)                                       #
# *********************************************************************#


# coding: utf-8

from __future__ import annotations

from datetime import date, datetime  # noqa: F401
from enum import Enum, IntEnum

import os
import re  # noqa: F401
from typing import Annotated, Any, Dict, List, Literal, Optional, Set, Type, Union, Callable, Iterable  # noqa: F401
from pathlib import Path
from typing import TypeVar
Model = TypeVar('Model', bound='BaseModel')
StrBytes = Union[str, bytes]

from pydantic import AnyUrl, BaseModel, EmailStr, Field, validator, root_validator, Extra,PrivateAttr  # noqa: F401
from pydantic import ValidationError
from pydantic.error_wrappers import ErrorWrapper
from pydantic.utils import ROOT_KEY
from json import encoder
from dnv_bladed_models.bladed_model import BladedModel
from dnv_bladed_models.nodes_at_heights import NodesAtHeights
from dnv_bladed_models.tower_node_loads import TowerNodeLoads
from dnv_bladed_models.tower_node_outputs import TowerNodeOutputs
class TowerOutputGroup_AxisSystemEnum(str, Enum):
    GLOBAL = "GLOBAL"

from .schema_helper import SchemaHelper
from .models_impl import *


class TowerOutputGroup(BladedModel):
    r"""
    A named tower output group.
    
    Not supported yet.
    
    Attributes
    ----------
    SelectedNodeLoads : TowerNodeLoads, Not supported yet
    
    AxisSystem : TowerOutputGroup_AxisSystemEnum, default='GLOBAL', Not supported yet
        The axis sytstem to be used for the outputs.
    
    AllConnectableNodes : TowerNodeOutputs, Not supported yet
    
    AllNodes : TowerNodeOutputs, Not supported yet
    
    NodesAtHeights : NodesAtHeights, Not supported yet
    
    Notes
    -----
    This model is not supported yet by the Bladed calculation engine.
    """
    SelectedNodeLoads: TowerNodeLoads = Field(alias="SelectedNodeLoads", default=None) # Not supported yet
    AxisSystem: TowerOutputGroup_AxisSystemEnum = Field(alias="AxisSystem", default=None) # Not supported yet
    AllConnectableNodes: TowerNodeOutputs = Field(alias="AllConnectableNodes", default=None) # Not supported yet
    AllNodes: TowerNodeOutputs = Field(alias="AllNodes", default=None) # Not supported yet
    NodesAtHeights: NodesAtHeights = Field(alias="NodesAtHeights", default=NodesAtHeights()) # Not supported yet

    _relative_schema_path = 'Components/Tower/TowerOutputGroupLibrary/TowerOutputGroup/TowerOutputGroup.json'
    _type_info = TypeInfo(
        set([]),
        set([]),
        set(['NodesAtHeights',]),
        None).merge(BladedModel._type_info)


    class Config:
        extra = Extra.forbid
        validate_assignment = True
        allow_population_by_field_name = True
        pass

    def _entity(self) -> bool:
        return True


TowerOutputGroup.update_forward_refs()



# *********************************************************************#
# Bladed Python Models API                                             #
# Copyright (c) DNV Services UK Limited (c) 2025. All rights reserved. #
# MIT License (see license file)                                       #
# *********************************************************************#


# coding: utf-8

from __future__ import annotations

from abc import ABC

from datetime import date, datetime  # noqa: F401
from enum import Enum, IntEnum

import os
import re  # noqa: F401
from typing import Annotated, Any, Dict, List, Literal, Optional, Set, Type, Union, Callable, Iterable  # noqa: F401
from pathlib import Path
from typing import TypeVar
Model = TypeVar('Model', bound='BaseModel')
StrBytes = Union[str, bytes]

from pydantic import AnyUrl, BaseModel, EmailStr, Field, validator, root_validator, Extra,PrivateAttr  # noqa: F401
from pydantic import ValidationError
from pydantic.error_wrappers import ErrorWrapper
from pydantic.utils import ROOT_KEY
from json import encoder
from dnv_bladed_models.bladed_model import BladedModel
from dnv_bladed_models.vector3_d import Vector3D
class AppliedLoad_AppliedLoadTypeEnum(str, Enum):
    BLADE_POINT_LOADING = "BladePointLoading"
    TOWER_POINT_LOADING = "TowerPointLoading"
    INSERT = "Insert"

from .schema_helper import SchemaHelper
from .models_impl import *


class AppliedLoad(BladedModel, ABC):
    r"""
    The common properties of a point loading time history.
    
    Attributes
    ----------
    AppliedLoadType : AppliedLoad_AppliedLoadTypeEnum
        Defines the specific type of model in use.
    
    StartTime : float
        The time into the simulation at which to start applying the loading (excluding the lead-in time).
    
    LoadingFilepath : str
        A filepath or URI containing one or six degree of loading data.  In the case of the six degrees of freedom, these will be applied in the component's coordinate system.  Where a single degree of freedom is provided, SingleDirectionLoading must also be specified.
    
    DirectionOfLoading : Vector3D
    
    OnComponentInAssembly : str, regex=^Assembly.(.+)$
        A qualified, dot-separated path to a component in the assembly tree to which to apply the force.  i.e. `Assembly.<name1>.<name2>`
    
    Notes
    -----
    
    This class is an abstraction, with the following concrete implementations:
        - BladePointLoading
        - AppliedLoadInsert
        - TowerPointLoading
    
    """
    AppliedLoadType: AppliedLoad_AppliedLoadTypeEnum = Field(alias="AppliedLoadType", default=None)
    StartTime: float = Field(alias="StartTime", default=None)
    LoadingFilepath: str = Field(alias="LoadingFilepath", default=None)
    DirectionOfLoading: Vector3D = Field(alias="DirectionOfLoading", default=None)
    OnComponentInAssembly: str = Field(alias="@OnComponentInAssembly", default=None, regex='^Assembly.(.+)$')

    _type_info = TypeInfo(
        set([]),
        set([]),
        set([]),
        None).merge(BladedModel._type_info)


AppliedLoad.update_forward_refs()



# *********************************************************************#
# Bladed Python Models API                                             #
# Copyright (c) DNV Services UK Limited (c) 2025. All rights reserved. #
# MIT License (see license file)                                       #
# *********************************************************************#


# coding: utf-8

from __future__ import annotations

from datetime import date, datetime  # noqa: F401
from enum import Enum, IntEnum

import os
import re  # noqa: F401
from typing import Annotated, Any, Dict, List, Literal, Optional, Set, Type, Union, Callable, Iterable  # noqa: F401
from pathlib import Path
from typing import TypeVar
Model = TypeVar('Model', bound='BaseModel')
StrBytes = Union[str, bytes]

from pydantic import AnyUrl, BaseModel, EmailStr, Field, validator, root_validator, Extra,PrivateAttr  # noqa: F401
from pydantic import ValidationError
from pydantic.error_wrappers import ErrorWrapper
from pydantic.utils import ROOT_KEY
from json import encoder
from dnv_bladed_models.foundation import Foundation

from .schema_helper import SchemaHelper
from .models_impl import *


class SimplifiedLinearFoundation(Foundation):
    r"""
    Foundation type ?.  This *may* be a super-simple foundation.
    
    Attributes
    ----------
    FoundationType : Literal['SimplifiedLinearFoundation'], default='SimplifiedLinearFoundation'
        Defines the specific type of Foundation model in use.  For a `SimplifiedLinearFoundation` object, this must always be set to a value of `SimplifiedLinearFoundation`.
    
    TranslationalStiffness : float
        Translational stiffness of foundation;KTRANS   Nm/rad Transitional stiffness of fopundation  Omit if no translation permited (TRANDOF is null).
    
    Mass : float, default=24000
        Foundation mass;FMASS       kg  Mass of foundation
    
    RotationalStiffness : float
        Rotational stiffness of foundation;KROT       N/m  Rotational stiffness of foundation  Omit if no translation permited (ROTDOF is null).
    
    RotationalInertia : float, default=380000
        Moment of inertia of foundation;FOUNDI     kgmÂ² Inertia of foundation mass about a horizontal axis at tower base
    
    Notes
    -----
    
    """
    FoundationType: Literal['SimplifiedLinearFoundation'] = Field(alias="FoundationType", default='SimplifiedLinearFoundation', allow_mutation=False, const=True) # type: ignore
    TranslationalStiffness: float = Field(alias="TranslationalStiffness", default=None)
    Mass: float = Field(alias="Mass", default=None)
    RotationalStiffness: float = Field(alias="RotationalStiffness", default=None)
    RotationalInertia: float = Field(alias="RotationalInertia", default=None)

    _relative_schema_path = 'Components/Tower/Foundation/SimplifiedLinearFoundation.json'
    _type_info = TypeInfo(
        set([]),
        set([]),
        set([]),
        'FoundationType').merge(Foundation._type_info)


    class Config:
        extra = Extra.forbid
        validate_assignment = True
        allow_population_by_field_name = True
        pass

    def _entity(self) -> bool:
        return True


SimplifiedLinearFoundation.update_forward_refs()



# *********************************************************************#
# Bladed Python Models API                                             #
# Copyright (c) DNV Services UK Limited (c) 2025. All rights reserved. #
# MIT License (see license file)                                       #
# *********************************************************************#


# coding: utf-8

from __future__ import annotations

from datetime import date, datetime  # noqa: F401
from enum import Enum, IntEnum

import os
import re  # noqa: F401
from typing import Annotated, Any, Dict, List, Literal, Optional, Set, Type, Union, Callable, Iterable  # noqa: F401
from pathlib import Path
from typing import TypeVar
Model = TypeVar('Model', bound='BaseModel')
StrBytes = Union[str, bytes]

from pydantic import AnyUrl, BaseModel, EmailStr, Field, validator, root_validator, Extra,PrivateAttr  # noqa: F401
from pydantic import ValidationError
from pydantic.error_wrappers import ErrorWrapper
from pydantic.utils import ROOT_KEY
from json import encoder
from dnv_bladed_models.component import Component
from dnv_bladed_models.interface_load_file import InterfaceLoadFile
from dnv_bladed_models.superelement_connectable_nodes import SuperelementConnectableNodes
class Superelement_WaveLoadingUnitsInFileEnum(str, Enum):
    N = "N"
    K_N = "kN"

from .schema_helper import SchemaHelper
from .models_impl import *


class Superelement(Component):
    r"""
    A structural superelement, mathematically representing a structural component.
    
    Not supported yet.
    
    Attributes
    ----------
    ComponentType : Literal['Superelement'], default='Superelement', Not supported yet
        Defines the specific type of Component model in use.  For a `Superelement` object, this must always be set to a value of `Superelement`.
    
    DefinitionFilepath : str, Not supported yet
        The filepath of URI to the superelement data.
    
    WaveLoadingFilepath : str, Not supported yet
        The filepath of URI to the accompanying wave data.
    
    WaveLoadingUnitsInFile : Superelement_WaveLoadingUnitsInFileEnum, default='kN', Not supported yet
        The units that were used when generating the wave load files.  The industry standard is to provide wave loads in kN and kNm.
    
    CoordinateTransformationMatrix : List[List[float]]
        A 6x6 matrix to transform one set of coordinates into another.
    
    InterfaceLoadFile : InterfaceLoadFile, Not supported yet
    
    ConnectableNodes : SuperelementConnectableNodes, Not supported yet
    
    Notes
    -----
    This model is not supported yet by the Bladed calculation engine.
    """
    ComponentType: Literal['Superelement'] = Field(alias="ComponentType", default='Superelement', allow_mutation=False, const=True) # Not supported yet # type: ignore
    DefinitionFilepath: str = Field(alias="DefinitionFilepath", default=None) # Not supported yet
    WaveLoadingFilepath: str = Field(alias="WaveLoadingFilepath", default=None) # Not supported yet
    WaveLoadingUnitsInFile: Superelement_WaveLoadingUnitsInFileEnum = Field(alias="WaveLoadingUnitsInFile", default=None) # Not supported yet
    CoordinateTransformationMatrix: List[List[float]] = Field(alias="CoordinateTransformationMatrix", default=list())
    InterfaceLoadFile: InterfaceLoadFile = Field(alias="InterfaceLoadFile", default=None) # Not supported yet
    ConnectableNodes: SuperelementConnectableNodes = Field(alias="ConnectableNodes", default=SuperelementConnectableNodes()) # Not supported yet

    _relative_schema_path = 'Components/Superelement/Superelement.json'
    _type_info = TypeInfo(
        set([]),
        set([]),
        set(['CoordinateTransformationMatrix','ConnectableNodes',]),
        'ComponentType').merge(Component._type_info)


    class Config:
        extra = Extra.forbid
        validate_assignment = True
        allow_population_by_field_name = True
        pass

    def _entity(self) -> bool:
        return True


Superelement.update_forward_refs()



# *********************************************************************#
# Bladed Python Models API                                             #
# Copyright (c) DNV Services UK Limited (c) 2025. All rights reserved. #
# MIT License (see license file)                                       #
# *********************************************************************#


# coding: utf-8

from __future__ import annotations

from abc import ABC

from datetime import date, datetime  # noqa: F401
from enum import Enum, IntEnum

import os
import re  # noqa: F401
from typing import Annotated, Any, Dict, List, Literal, Optional, Set, Type, Union, Callable, Iterable  # noqa: F401
from pathlib import Path
from typing import TypeVar
Model = TypeVar('Model', bound='BaseModel')
StrBytes = Union[str, bytes]

from pydantic import AnyUrl, BaseModel, EmailStr, Field, validator, root_validator, Extra,PrivateAttr  # noqa: F401
from pydantic import ValidationError
from pydantic.error_wrappers import ErrorWrapper
from pydantic.utils import ROOT_KEY
from json import encoder
from dnv_bladed_models.bladed_model import BladedModel
from dnv_bladed_models.velocity_range import VelocityRange
class BladeStabilityAnalysisModeOfOperation_BladeStabilityAnalysisModeOfOperationTypeEnum(str, Enum):
    IDLING = "Idling"
    PARKED = "Parked"
    INSERT = "Insert"

from .schema_helper import SchemaHelper
from .models_impl import *


class BladeStabilityAnalysisModeOfOperation(BladedModel, ABC):
    r"""
    
    
    Not supported yet.
    
    Attributes
    ----------
    BladeStabilityAnalysisModeOfOperationType : BladeStabilityAnalysisModeOfOperation_BladeStabilityAnalysisModeOfOperationTypeEnum, Not supported yet
        Defines the specific type of model in use.
    
    PitchAngle : float, default=0, Not supported yet
        The constant pitch angle used in both parked and free spin case.  This value is only used for blade stability analysis and not for model linearisation or Campbell diagram.  Blade set angle or pitch angle imbalances are ignored, and any pitch limits are not used.
    
    WindSpeedRange : VelocityRange
    
    Notes
    -----
    This model is not supported yet by the Bladed calculation engine.
    This class is an abstraction, with the following concrete implementations:
        - BladeStabilityAnalysisIdling
        - BladeStabilityAnalysisModeOfOperationInsert
        - BladeStabilityAnalysisParked
    
    """
    BladeStabilityAnalysisModeOfOperationType: BladeStabilityAnalysisModeOfOperation_BladeStabilityAnalysisModeOfOperationTypeEnum = Field(alias="BladeStabilityAnalysisModeOfOperationType", default=None) # Not supported yet
    PitchAngle: float = Field(alias="PitchAngle", default=None) # Not supported yet
    WindSpeedRange: VelocityRange = Field(alias="WindSpeedRange", default=None)

    _type_info = TypeInfo(
        set([]),
        set([]),
        set([]),
        None).merge(BladedModel._type_info)


BladeStabilityAnalysisModeOfOperation.update_forward_refs()



# *********************************************************************#
# Bladed Python Models API                                             #
# Copyright (c) DNV Services UK Limited (c) 2025. All rights reserved. #
# MIT License (see license file)                                       #
# *********************************************************************#


# coding: utf-8

from __future__ import annotations

from datetime import date, datetime  # noqa: F401
from enum import Enum, IntEnum

import os
import re  # noqa: F401
from typing import Annotated, Any, Dict, List, Literal, Optional, Set, Type, Union, Callable, Iterable  # noqa: F401
from pathlib import Path
from typing import TypeVar
Model = TypeVar('Model', bound='BaseModel')
StrBytes = Union[str, bytes]

from pydantic import AnyUrl, BaseModel, EmailStr, Field, validator, root_validator, Extra,PrivateAttr  # noqa: F401
from pydantic import ValidationError
from pydantic.error_wrappers import ErrorWrapper
from pydantic.utils import ROOT_KEY
from json import encoder
from dnv_bladed_models.initial_condition import InitialCondition

from .schema_helper import SchemaHelper
from .models_impl import *


class InitialRotorSpeed(InitialCondition):
    r"""
    The rotor speed.  If there is only one rotor or all rotors have the same initial speed, then AssemblyReference can be omitted.
    
    Attributes
    ----------
    InitialConditionType : Literal['InitialRotorSpeed'], default='InitialRotorSpeed'
        Defines the specific type of InitialCondition model in use.  For a `InitialRotorSpeed` object, this must always be set to a value of `InitialRotorSpeed`.
    
    RotorSpeed : float
        The rotor speed to use during initial conditions.  If omitted, Bladed will attempt to determine a combination of generator torque and pitch angle that would result in the rotor operating at a constant speed.  Once the simulation begins, environmental and control changes can result in this speed changing unless MaintainInitialValueThroughoutSimulation is true.
    
    MaintainInitialValueThroughoutSimulation : bool, default=False
        If true, the rotor speed will be prescribed to maintain whatever speed was determined during initial conditions.  This cannot be overcome by aerodynamic loads or other physical forces, and the generator torque will become irrelevant.
    
    Notes
    -----
    
    """
    InitialConditionType: Literal['InitialRotorSpeed'] = Field(alias="InitialConditionType", default='InitialRotorSpeed', allow_mutation=False, const=True) # type: ignore
    RotorSpeed: float = Field(alias="RotorSpeed", default=None)
    MaintainInitialValueThroughoutSimulation: bool = Field(alias="MaintainInitialValueThroughoutSimulation", default=None)

    _relative_schema_path = 'TimeDomainSimulation/InitialCondition/InitialRotorSpeed.json'
    _type_info = TypeInfo(
        set([]),
        set([]),
        set([]),
        'InitialConditionType').merge(InitialCondition._type_info)


    class Config:
        extra = Extra.forbid
        validate_assignment = True
        allow_population_by_field_name = True
        pass

    def _entity(self) -> bool:
        return True


InitialRotorSpeed.update_forward_refs()



# *********************************************************************#
# Bladed Python Models API                                             #
# Copyright (c) DNV Services UK Limited (c) 2025. All rights reserved. #
# MIT License (see license file)                                       #
# *********************************************************************#


# coding: utf-8

from __future__ import annotations

from abc import ABC

from datetime import date, datetime  # noqa: F401
from enum import Enum, IntEnum

import os
import re  # noqa: F401
from typing import Annotated, Any, Dict, List, Literal, Optional, Set, Type, Union, Callable, Iterable  # noqa: F401
from pathlib import Path
from typing import TypeVar
Model = TypeVar('Model', bound='BaseModel')
StrBytes = Union[str, bytes]

from pydantic import AnyUrl, BaseModel, EmailStr, Field, validator, root_validator, Extra,PrivateAttr  # noqa: F401
from pydantic import ValidationError
from pydantic.error_wrappers import ErrorWrapper
from pydantic.utils import ROOT_KEY
from json import encoder
from dnv_bladed_models.bladed_model import BladedModel
class DynamicStall_DynamicStallTypeEnum(str, Enum):
    COMPRESSIBLE_BEDDOES_LEISHMAN_MODEL = "CompressibleBeddoesLeishmanModel"
    IAG_MODEL = "IAGModel"
    INCOMPRESSIBLE_BEDDOES_LEISHMAN_MODEL = "IncompressibleBeddoesLeishmanModel"
    OYE_MODEL = "OyeModel"
    INSERT = "Insert"

from .schema_helper import SchemaHelper
from .models_impl import *


class DynamicStall(BladedModel, ABC):
    r"""
    The common properties of all of the dynamic stall models.
    
    Attributes
    ----------
    DynamicStallType : DynamicStall_DynamicStallTypeEnum
        Defines the specific type of model in use.
    
    UseDynamicPitchingMomentCoefficient : bool, default=True
        If true, the dynamic pitching moment coefficient will be used.  This option is enabled by default.  It is not recommended to disable this option for blades with a torsional degree of freedom because the so-called 'pitch- rate damping' term of the moment coefficient is typically important to keep the blade torsional mode stable.
    
    StartingRadius : float, default=0
        The fraction of the radius outboard of which the dynamic stall model will be used. A value of 0.0 means that dynamic stall is applied from the blade root.
    
    EndingRadius : float, default=0.95
        The fraction of the radius outboard of which the dynamic stall model will be switched off. A value of 1.0 means that dynamic stall is applied until the blade tip.
    
    SeparationTimeConstant : float, default=3
        A dimensionless time constant, given in terms of the time taken to travel half a chord. It defines the lag in the movement of the separation point due to unsteady pressure and boundary layer response.
    
    UseLinearFitGradientMethod : bool, default=True
        If true, the linear fit polar gradient is used to reconstruct the inviscid polar data. The fit is performed only within the linear polar regime that is searched automatically between the zero lift AoA to AoA = 7 deg. This approach is more suitable for polar data sets where the lift coefficient slope is not straight around 0 deg angle of attack. It is recommended to activate this option for more accurate computations. This option is turned on by default.
    
    Notes
    -----
    
    This class is an abstraction, with the following concrete implementations:
        - CompressibleBeddoesLeishmanModel
        - IAGModel
        - IncompressibleBeddoesLeishmanModel
        - DynamicStallInsert
        - OyeModel
    
    """
    DynamicStallType: DynamicStall_DynamicStallTypeEnum = Field(alias="DynamicStallType", default=None)
    UseDynamicPitchingMomentCoefficient: bool = Field(alias="UseDynamicPitchingMomentCoefficient", default=None)
    StartingRadius: float = Field(alias="StartingRadius", default=None)
    EndingRadius: float = Field(alias="EndingRadius", default=None)
    SeparationTimeConstant: float = Field(alias="SeparationTimeConstant", default=None)
    UseLinearFitGradientMethod: bool = Field(alias="UseLinearFitGradientMethod", default=None)

    _type_info = TypeInfo(
        set([]),
        set([]),
        set([]),
        None).merge(BladedModel._type_info)


DynamicStall.update_forward_refs()



# *********************************************************************#
# Bladed Python Models API                                             #
# Copyright (c) DNV Services UK Limited (c) 2025. All rights reserved. #
# MIT License (see license file)                                       #
# *********************************************************************#


# coding: utf-8

from __future__ import annotations

from datetime import date, datetime  # noqa: F401
from enum import Enum, IntEnum

import os
import re  # noqa: F401
from typing import Annotated, Any, Dict, List, Literal, Optional, Set, Type, Union, Callable, Iterable  # noqa: F401
from pathlib import Path
from typing import TypeVar
Model = TypeVar('Model', bound='BaseModel')
StrBytes = Union[str, bytes]

from pydantic import AnyUrl, BaseModel, EmailStr, Field, validator, root_validator, Extra,PrivateAttr  # noqa: F401
from pydantic import ValidationError
from pydantic.error_wrappers import ErrorWrapper
from pydantic.utils import ROOT_KEY
from json import encoder
from dnv_bladed_models.bladed_model import BladedModel

from .schema_helper import SchemaHelper
from .models_impl import *


class AerodynamicProperties(BladedModel):
    r"""
    The aerodynamic properties of the blade cross-section.  These are defined in the QuarterChordCoordinateSystem, which specifies the position and orientation of the aerofoil.
    
    Attributes
    ----------
    Chord : float
        The chord of the blade at this cross-section, measured from leading to trailing edge.  The cross-section will be positioned so that the aerofoil's quarter-chord position coincides with the origin of the QuarterChordCoordinateSystem.
    
    ThicknessToChordRatio : float
        The ratio between the thickness and the chord length at the cross-section.  This would be 1.0 for a cylinder, and in the order of 0.1 (or 10%) for a typical slender aerofoil section.
    
    AerodynamicData : str, regex=^(AerofoilLibrary|InterpolatedAerofoilLibrary|AileronAerofoilLibrary).(.+)$
        A reference to an aerofoil, interpolated aerofoil, or aileron aerofoil to use for this cross-section.  The value must be the key of an aerofoil definition in either the AerofoilLibrary, InterpolatedAerofoilLibrary or AileronAerofoilLibrary.  i.e.  `AerofoilLibrary.<key>` or `InterpolatedAerofoilLibrary.<key>` or `AileronAerofoilLibrary.<key>`
    
    Notes
    -----
    
    """
    Chord: float = Field(alias="Chord", default=None)
    ThicknessToChordRatio: float = Field(alias="ThicknessToChordRatio", default=None)
    AerodynamicData: str = Field(alias="@AerodynamicData", default=None, regex='^(AerofoilLibrary|InterpolatedAerofoilLibrary|AileronAerofoilLibrary).(.+)$')

    _relative_schema_path = 'Components/Blade/CrossSection/AerodynamicProperties/AerodynamicProperties.json'
    _type_info = TypeInfo(
        set([]),
        set([]),
        set([]),
        None).merge(BladedModel._type_info)


    class Config:
        extra = Extra.forbid
        validate_assignment = True
        allow_population_by_field_name = True
        pass

    def _entity(self) -> bool:
        return True


AerodynamicProperties.update_forward_refs()
